# -*- coding: utf-8 -*-
"""SQLite parser plugin for MacOS and iOS iMessage database files."""

from dfdatetime import cocoa_time as dfdatetime_cocoa_time
from dfdatetime import definitions as dfdatetime_definitions

from plaso.containers import events
from plaso.lib import definitions
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class IMessageEventData(events.EventData):
  """iMessage and SMS event data.

  Attributes:
    attachment_location (str): location of the attachment.
    client_version (int): client version.
    creation_time (dfdatetime.DateTimeValues): date and time the message
        was created.
    imessage_id (str): mobile number or email address the message was sent
        to or received from.
    message_type (int): value to indicate the message was sent (1) or
        received (0).
    offset (str): identifier of the row, from which the event data was
        extracted.
    query (str): SQL query that was used to obtain the event data.
    read_receipt (bool): True if the message read receipt was received.
    service (str): service, which is either SMS or iMessage.
    text (str): content of the message.
  """

  DATA_TYPE = 'imessage:event:chat'

  def __init__(self):
    """Initializes event data."""
    super(IMessageEventData, self).__init__(data_type=self.DATA_TYPE)
    self.attachment_location = None
    self.client_version = None
    self.creation_time = None
    self.imessage_id = None
    self.message_type = None
    self.offset = None
    self.query = None
    self.read_receipt = None
    self.service = None
    self.text = None


class IMessagePlugin(interface.SQLitePlugin):
  """SQLite parser plugin for MacOS and iOS iMessage database files.

  The iMessage database file is typically stored in chat.db or sms.db.
  """

  NAME = 'imessage'
  DATA_FORMAT = 'MacOS and iOS iMessage database (chat.db, sms.db) file'

  REQUIRED_STRUCTURE = {
      '_SqliteDatabaseProperties': frozenset([
          'key', 'value']),
      'message': frozenset([
          'date', 'ROWID', 'is_read', 'is_from_me', 'service', 'text',
          'handle_id']),
      'handle': frozenset([
          'id', 'ROWID']),
      'attachment': frozenset([
          'filename', 'ROWID']),
      'message_attachment_join': frozenset([
          'message_id', 'attachment_id'])}

  QUERIES = [
      ('SELECT message.date, message.ROWID, handle.id AS imessage_id, '
       'message.is_read AS read_receipt, message.is_from_me AS message_type, '
       'message.service, attachment.filename AS attachment_location, '
       'message.text FROM message JOIN handle '
       'ON handle.ROWID = message.handle_id '
       'LEFT OUTER JOIN message_attachment_join AS maj '
       'ON message.ROWID = maj.message_id LEFT OUTER JOIN attachment '
       'ON maj.attachment_id = attachment.ROWID', 'ParseMessageRow')]

  _CLIENT_VERSION_QUERY = (
      'SELECT key, value FROM _SqliteDatabaseProperties '
      'WHERE key = "_ClientVersion"')

  def _GetClientVersion(self, cache, database):
    """Retrieves the client version.

    Args:
      cache (SQLiteCache): cache.
      database (SQLiteDatabase): database.

    Returns:
      int: client version or None if the client version cannot be determined.
    """
    cache_results = cache.GetResults('client_version')
    if not cache_results:
      query_result = database.Query(self._CLIENT_VERSION_QUERY)

      cache.CacheQueryResults(
          query_result, 'client_version', 'key', ('value', ))
      cache_results = cache.GetResults('client_version')

    client_version = cache_results.get('_ClientVersion', [])[0]

    if isinstance(client_version, str):
      try:
        client_version = int(client_version, 10)
        cache_results['_ClientVersion'] = [client_version]
      except (ValueError, TypeError):
        return None

    return client_version

  def _GetDateTimeRowValue(self, query_hash, row, value_name):
    """Retrieves a date and time value from the row.

    Args:
      query_hash (int): hash of the query, that uniquely identifies the query
          that produced the row.
      row (sqlite3.Row): row.
      value_name (str): name of the value.

    Returns:
      dfdatetime.CocoaTime: date and time value or None if not available.
    """
    timestamp = self._GetRowValue(query_hash, row, value_name)
    if timestamp is None:
      return None

    precision = dfdatetime_definitions.PRECISION_1_SECOND

    # In current versions the timestamp is stored in nanoseconds.
    # Note that a Cocoa timestamp of 1000000000 is somewhere in 2032 and
    # the timestamp apprears to have been changes around 2017.
    if (timestamp < -definitions.NANOSECONDS_PER_SECOND or
        timestamp > definitions.NANOSECONDS_PER_SECOND):
      timestamp /= definitions.NANOSECONDS_PER_SECOND
      precision = dfdatetime_definitions.PRECISION_1_NANOSECOND

    return dfdatetime_cocoa_time.CocoaTime(
        precision=precision, timestamp=timestamp)

  def ParseMessageRow(
      self, parser_mediator, query, row, cache=None, database=None,
      **unused_kwargs):
    """Parses a message row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      query (str): query that created the row.
      row (sqlite3.Row): row.
      cache (SQLiteCache): cache which contains cached results from querying
          the visits and urls tables.
      database (Optional[SQLiteDatabase]): database.
    """
    query_hash = hash(query)

    event_data = IMessageEventData()
    event_data.attachment_location = self._GetRowValue(
        query_hash, row, 'attachment_location')
    event_data.client_version = self._GetClientVersion(cache, database)
    event_data.creation_time = self._GetDateTimeRowValue(
        query_hash, row, 'date')
    event_data.imessage_id = self._GetRowValue(query_hash, row, 'imessage_id')
    event_data.message_type = self._GetRowValue(query_hash, row, 'message_type')
    event_data.offset = self._GetRowValue(query_hash, row, 'ROWID')
    event_data.query = query
    event_data.read_receipt = self._GetRowValue(query_hash, row, 'read_receipt')
    event_data.service = self._GetRowValue(query_hash, row, 'service')
    event_data.text = self._GetRowValue(query_hash, row, 'text')

    parser_mediator.ProduceEventData(event_data)


sqlite.SQLiteParser.RegisterPlugin(IMessagePlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the iPod, iPad and iPhone storage plist plugin."""

import unittest

from plaso.parsers.plist_plugins import ipod

from tests.parsers.plist_plugins import test_lib


class TestIPodPlugin(test_lib.PlistPluginTestCase):
  """Tests for the iPod, iPad and iPhone storage plist plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plist_name = 'com.apple.iPod.plist'

    plugin = ipod.IPodPlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 4)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'ipod:device:entry',
        'device_class': 'iPhone',
        'device_identifier': '4C6F6F6E65000000',
        'family_identifier': 10016,
        'firmware_version': '7.0',
        'imei': '012345678901234',
        'last_connected_time': '2013-10-09T19:27:54.000000+00:00',
        'serial_number': '526F676572',
        'use_count': 1}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 3)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Windows EventLog resources database reader."""

import collections
import os
import sqlite3

from plaso.engine import path_helper
from plaso.helpers.windows import languages
from plaso.helpers.windows import resource_files
from plaso.output import logger


class Sqlite3DatabaseFile(object):
  """Class that defines a sqlite3 database file."""

  _HAS_TABLE_QUERY = (
      'SELECT name FROM sqlite_master '
      'WHERE type = "table" AND name = "{0:s}"')

  def __init__(self):
    """Initializes the database file object."""
    super(Sqlite3DatabaseFile, self).__init__()
    self._connection = None
    self._cursor = None
    self.filename = None
    self.read_only = None

  def Close(self):
    """Closes the database file.

    Raises:
      RuntimeError: if the database is not opened.
    """
    if not self._connection:
      raise RuntimeError('Cannot close database not opened.')

    # We need to run commit or not all data is stored in the database.
    self._connection.commit()
    self._connection.close()

    self._connection = None
    self._cursor = None
    self.filename = None
    self.read_only = None

  def HasTable(self, table_name):
    """Determines if a specific table exists.

    Args:
      table_name (str): table name.

    Returns:
      bool: True if the table exists.

    Raises:
      RuntimeError: if the database is not opened.
    """
    if not self._connection:
      raise RuntimeError(
          'Cannot determine if table exists database not opened.')

    sql_query = self._HAS_TABLE_QUERY.format(table_name)

    self._cursor.execute(sql_query)
    if self._cursor.fetchone():
      return True

    return False

  def GetValues(self, table_names, column_names, condition):
    """Retrieves values from a table.

    Args:
      table_names (list[str]): table names.
      column_names (list[str]): column names.
      condition (str): query condition such as
          "log_source == 'Application Error'".

    Yields:
      sqlite3.row: row.

    Raises:
      RuntimeError: if the database is not opened.
    """
    if not self._connection:
      raise RuntimeError('Cannot retrieve values database not opened.')

    if condition:
      condition = ' WHERE {0:s}'.format(condition)

    sql_query = 'SELECT {1:s} FROM {0:s}{2:s}'.format(
        ', '.join(table_names), ', '.join(column_names), condition)

    self._cursor.execute(sql_query)

    # TODO: have a look at https://docs.python.org/2/library/
    # sqlite3.html#sqlite3.Row.
    for row in self._cursor:
      yield {
          column_name: row[column_index]
          for column_index, column_name in enumerate(column_names)}

  def Open(self, filename, read_only=False):
    """Opens the database file.

    Args:
      filename (str): filename of the database.
      read_only (Optional[bool]): True if the database should be opened in
          read-only mode. Since sqlite3 does not support a real read-only
          mode we fake it by only permitting SELECT queries.

    Returns:
      bool: True if successful.

    Raises:
      RuntimeError: if the database is already opened.
    """
    if self._connection:
      raise RuntimeError('Cannot open database already opened.')

    self.filename = filename
    self.read_only = read_only

    try:
      self._connection = sqlite3.connect(filename)
    except sqlite3.OperationalError:
      return False

    if not self._connection:
      return False

    self._cursor = self._connection.cursor()
    if not self._cursor:
      return False

    return True


class WinevtResourcesSqlite3DatabaseReader(object):
  """Windows EventLog resources SQLite database reader."""

  def __init__(self):
    """Initializes a Windows EventLog resources SQLite database reader."""
    super(WinevtResourcesSqlite3DatabaseReader, self).__init__()
    self._database_file = Sqlite3DatabaseFile()
    self._resouce_file_helper = resource_files.WindowsResourceFileHelper
    self._string_format = 'wrc'

  def _GetEventLogProviderKey(self, log_source):
    """Retrieves the EventLog provider key.

    Args:
      log_source (str): EventLog source.

    Returns:
      str: EventLog provider key or None if not available.

    Raises:
      RuntimeError: if more than one value is found in the database.
    """
    table_names = ['event_log_providers']
    column_names = ['event_log_provider_key']
    condition = 'log_source == "{0:s}"'.format(log_source)

    values_list = list(self._database_file.GetValues(
        table_names, column_names, condition))

    number_of_values = len(values_list)
    if number_of_values == 0:
      return None

    if number_of_values == 1:
      values = values_list[0]
      return values['event_log_provider_key']

    raise RuntimeError('More than one value found in database.')

  def _GetMessage(self, message_file_key, lcid, message_identifier):
    """Retrieves a specific message from a specific message table.

    Args:
      message_file_key (int): message file key.
      lcid (int): language code identifier (LCID).
      message_identifier (int): message identifier.

    Returns:
      str: message string or None if not available.

    Raises:
      RuntimeError: if more than one value is found in the database.
    """
    table_name = 'message_table_{0:d}_0x{1:08x}'.format(message_file_key, lcid)

    has_table = self._database_file.HasTable(table_name)
    if not has_table:
      return None

    column_names = ['message_string']
    condition = 'message_identifier == "0x{0:08x}"'.format(message_identifier)

    values = list(self._database_file.GetValues(
        [table_name], column_names, condition))

    number_of_values = len(values)
    if number_of_values == 0:
      return None

    if number_of_values == 1:
      return values[0]['message_string']

    raise RuntimeError('More than one value found in database.')

  def _GetMessageFileKeys(self, event_log_provider_key):
    """Retrieves the message file keys.

    Args:
      event_log_provider_key (int): EventLog provider key.

    Yields:
      int: message file key.
    """
    table_names = ['message_file_per_event_log_provider']
    column_names = ['message_file_key']
    condition = 'event_log_provider_key == {0:d}'.format(
        event_log_provider_key)

    generator = self._database_file.GetValues(
        table_names, column_names, condition)
    for values in generator:
      yield values['message_file_key']

  def Close(self):
    """Closes the database reader object."""
    self._database_file.Close()

  def GetMessage(self, log_source, lcid, message_identifier):
    """Retrieves a specific message for a specific EventLog source.

    Args:
      log_source (str): EventLog source.
      lcid (int): language code identifier (LCID).
      message_identifier (int): message identifier.

    Returns:
      str: message string or None if not available.
    """
    event_log_provider_key = self._GetEventLogProviderKey(log_source)
    if not event_log_provider_key:
      return None

    generator = self._GetMessageFileKeys(event_log_provider_key)
    if not generator:
      return None

    message_string = None
    for message_file_key in generator:
      message_string = self._GetMessage(
          message_file_key, lcid, message_identifier)

      if message_string:
        break

    if self._string_format == 'wrc':
      message_string = self._resouce_file_helper.FormatMessageStringInPEP3101(
          message_string)

    return message_string

  def GetMetadataAttribute(self, attribute_name):
    """Retrieves the metadata attribute.

    Args:
      attribute_name (str): name of the metadata attribute.

    Returns:
      str: the metadata attribute or None.

    Raises:
      RuntimeError: if more than one value is found in the database.
    """
    table_name = 'metadata'

    has_table = self._database_file.HasTable(table_name)
    if not has_table:
      return None

    column_names = ['value']
    condition = 'name == "{0:s}"'.format(attribute_name)

    values = list(self._database_file.GetValues(
        [table_name], column_names, condition))

    number_of_values = len(values)
    if number_of_values == 0:
      return None

    if number_of_values == 1:
      return values[0]['value']

    raise RuntimeError('More than one value found in database.')

  def Open(self, filename):
    """Opens the database reader object.

    Args:
      filename (str): filename of the database.

    Returns:
      bool: True if successful.

    Raises:
      RuntimeError: if the version or string format of the database
          is not supported.
    """
    if not self._database_file.Open(filename, read_only=True):
      return False

    version = self.GetMetadataAttribute('version')
    if not version or version != '20150315':
      raise RuntimeError('Unsupported version: {0:s}'.format(version))

    string_format = self.GetMetadataAttribute('string_format')
    if not string_format:
      string_format = 'wrc'

    if string_format not in ('pep3101', 'wrc'):
      raise RuntimeError('Unsupported string format: {0:s}'.format(
          string_format))

    self._string_format = string_format
    return True


class WinevtResourcesHelper(object):
  """Windows EventLog resources helper."""

  # LCID 0x0409 is en-US.
  DEFAULT_LCID = 0x0409

  # The maximum number of cached message strings
  _MAXIMUM_CACHED_MESSAGE_STRINGS = 32 * 1024

  _WINEVT_RC_DATABASE = 'winevt-rc.db'

  def __init__(self, storage_reader, data_location, lcid):
    """Initializes Windows EventLog resources helper.

    Args:
      storage_reader (StorageReader): storage reader.
      data_location (str): data location of the winevt-rc database.
      lcid (int): Windows Language Code Identifier (LCID).
    """
    language_tag = languages.WindowsLanguageHelper.GetLanguageTagForLCID(
        lcid or self.DEFAULT_LCID)

    super(WinevtResourcesHelper, self).__init__()
    self._data_location = data_location
    self._environment_variables = None
    self._language_tag = language_tag.lower()
    self._lcid = lcid or self.DEFAULT_LCID
    self._message_string_cache = collections.OrderedDict()
    self._storage_reader = storage_reader
    self._windows_eventlog_message_files = None
    self._windows_eventlog_providers = None
    self._winevt_database_reader = None

  def _CacheMessageString(
      self, provider_identifier, log_source, message_identifier,
      event_version, message_string):
    """Caches a specific message string.

    Args:
      provider_identifier (str): EventLog provider identifier.
      log_source (str): EventLog source, such as "Application Error".
      message_identifier (int): message identifier.
      event_version (int): event version or None if not set.
      message_string (str): message string.
    """
    if len(self._message_string_cache) >= self._MAXIMUM_CACHED_MESSAGE_STRINGS:
      self._message_string_cache.popitem(last=True)

    if provider_identifier:
      lookup_key = '{0:s}:0x{1:08x}'.format(
          provider_identifier, message_identifier)
      if event_version is not None:
        lookup_key = '{0:s}:{1:d}'.format(lookup_key, event_version)
      self._message_string_cache[lookup_key] = message_string
      self._message_string_cache.move_to_end(lookup_key, last=False)

    if log_source:
      lookup_key = '{0:s}:0x{1:08x}'.format(log_source, message_identifier)
      if event_version is not None:
        lookup_key = '{0:s}:{1:d}'.format(lookup_key, event_version)
      self._message_string_cache[lookup_key] = message_string
      self._message_string_cache.move_to_end(lookup_key, last=False)

  def _GetCachedMessageString(
      self, provider_identifier, log_source, message_identifier, event_version):
    """Retrieves a specific cached message string.

    Args:
      provider_identifier (str): EventLog provider identifier.
      log_source (str): EventLog source, such as "Application Error".
      message_identifier (int): message identifier.
      event_version (int): event version or None if not set.

    Returns:
      str: message string or None if not available.
    """
    lookup_key = None
    message_string = None

    if provider_identifier:
      lookup_key = '{0:s}:0x{1:08x}'.format(
          provider_identifier, message_identifier)
      if event_version is not None:
        lookup_key = '{0:s}:{1:d}'.format(lookup_key, event_version)
      message_string = self._message_string_cache.get(lookup_key, None)

    if not message_string and log_source:
      lookup_key = '{0:s}:0x{1:08x}'.format(log_source, message_identifier)
      if event_version is not None:
        lookup_key = '{0:s}:{1:d}'.format(lookup_key, event_version)
      message_string = self._message_string_cache.get(lookup_key, None)

    if message_string:
      self._message_string_cache.move_to_end(lookup_key, last=False)

    return message_string

  def _GetWinevtRcDatabaseReader(self):
    """Opens the Windows EventLog resource database reader.

    Returns:
      WinevtResourcesSqlite3DatabaseReader: Windows EventLog resource
          database reader or None.
    """
    if not self._winevt_database_reader and self._data_location:
      logger.warning((
          'Falling back to {0:s}. Please make sure the Windows EventLog '
          'message strings in the database correspond to those in the '
          'EventLog files.').format(self._WINEVT_RC_DATABASE))

      database_path = os.path.join(
          self._data_location, self._WINEVT_RC_DATABASE)
      if not os.path.isfile(database_path):
        return None

      self._winevt_database_reader = WinevtResourcesSqlite3DatabaseReader()
      if not self._winevt_database_reader.Open(database_path):
        self._winevt_database_reader = None

    return self._winevt_database_reader

  def _GetWinevtRcDatabaseMessageString(self, log_source, message_identifier):
    """Retrieves a specific Windows EventLog resource database message string.

    Args:
      log_source (str): EventLog source, such as "Application Error".
      message_identifier (int): message identifier.

    Returns:
      str: message string or None if not available.
    """
    database_reader = self._GetWinevtRcDatabaseReader()
    if not database_reader:
      return None

    if self._lcid != self.DEFAULT_LCID:
      message_string = database_reader.GetMessage(
          log_source, self._lcid, message_identifier)
      if message_string:
        return message_string

    return database_reader.GetMessage(
        log_source, self.DEFAULT_LCID, message_identifier)

  def _ReadEnvironmentVariables(self, storage_reader):
    """Reads the environment variables.

    Args:
      storage_reader (StorageReader): storage reader.
    """
    # TODO: get environment variables related to the source.
    self._environment_variables = list(storage_reader.GetAttributeContainers(
        'environment_variable'))

  def _ReadWindowsEventLogMessageFiles(self, storage_reader):
    """Reads the Windows EventLog message files.

    Args:
      storage_reader (StorageReader): storage reader.
    """
    # TODO: get windows eventlog message files related to the source.
    self._windows_eventlog_message_files = {}
    if storage_reader.HasAttributeContainers('windows_eventlog_message_file'):
      for message_file in storage_reader.GetAttributeContainers(
          'windows_eventlog_message_file'):
        path, filename = path_helper.PathHelper.GetWindowsSystemPath(
            message_file.path, self._environment_variables)

        lookup_path = '\\'.join([path, filename]).lower()
        message_file_identifier = message_file.GetIdentifier()
        self._windows_eventlog_message_files[lookup_path] = (
            message_file_identifier)

  def _ReadWindowsEventLogMessageString(
      self, storage_reader, provider_identifier, log_source,
      message_identifier, event_version):
    """Reads an Windows EventLog message string.

    Args:
      storage_reader (StorageReader): storage reader.
      provider_identifier (str): EventLog provider identifier.
      log_source (str): EventLog source, such as "Application Error".
      message_identifier (int): message identifier.
      event_version (int): event version or None if not set.

    Returns:
      str: message string or None if not available.
    """
    if self._environment_variables is None:
      self._ReadEnvironmentVariables(storage_reader)

    if self._windows_eventlog_providers is None:
      self._ReadWindowsEventLogProviders(storage_reader)

    if self._windows_eventlog_message_files is None:
      self._ReadWindowsEventLogMessageFiles(storage_reader)

    provider = None

    if provider_identifier:
      lookup_key = provider_identifier.lower()
      provider = self._windows_eventlog_providers.get(lookup_key, None)

    if not provider:
      lookup_key = log_source.lower()
      provider = self._windows_eventlog_providers.get(lookup_key, None)

    provider = self._windows_eventlog_providers.get(lookup_key, None)
    if not provider:
      return None

    if not storage_reader.HasAttributeContainers(
        'windows_eventlog_message_string'):
      return None

    # Map the event identifier to a message identifier as defined by the
    # WEVT_TEMPLATE event definition.
    if provider_identifier and storage_reader.HasAttributeContainers(
        'windows_wevt_template_event'):
      # TODO: add message_file_identifiers to filter_expression
      filter_expression = (
          'provider_identifier == "{0:s}" and identifier == {1:d}').format(
              provider_identifier, message_identifier)
      if event_version is not None:
        filter_expression = '{0:s} and version == {1:d}'.format(
            filter_expression, event_version)

      for event_definition in storage_reader.GetAttributeContainers(
          'windows_wevt_template_event', filter_expression=filter_expression):
        logger.debug(
            'Message: 0x{0:08x} of provider: {1:s} maps to: 0x{2:08x}'.format(
                message_identifier, provider_identifier,
                event_definition.message_identifier))
        message_identifier = event_definition.message_identifier
        break

    message_file_identifiers = []
    for windows_path in provider.event_message_files or []:
      path, filename = path_helper.PathHelper.GetWindowsSystemPath(
          windows_path, self._environment_variables)

      lookup_path = '\\'.join([path, filename]).lower()
      message_file_identifier = self._windows_eventlog_message_files.get(
          lookup_path, None)
      if message_file_identifier:
        message_file_identifier = message_file_identifier.CopyToString()
        message_file_identifiers.append(message_file_identifier)

      mui_filename = '{0:s}.mui'.format(filename)
      lookup_path = '\\'.join([path, self._language_tag, mui_filename]).lower()
      message_file_identifier = self._windows_eventlog_message_files.get(
          lookup_path, None)
      if message_file_identifier:
        message_file_identifier = message_file_identifier.CopyToString()
        message_file_identifiers.append(message_file_identifier)

    if not message_file_identifiers:
      logger.warning(
          'No message file for message: 0x{0:08x} of provider: {1:s}'.format(
              message_identifier, lookup_key))
      return None

    message_strings = []
    # TODO: add message_file_identifiers to filter_expression
    filter_expression = (
        'language_identifier == {0:d} and '
        'message_identifier == {1:d}').format(
            self._lcid, message_identifier)

    for message_string in storage_reader.GetAttributeContainers(
        'windows_eventlog_message_string',
        filter_expression=filter_expression):
      identifier = message_string.GetMessageFileIdentifier()
      identifier = identifier.CopyToString()
      if identifier in message_file_identifiers:
        message_strings.append(message_string)

    if not message_strings:
      logger.warning((
          'No message string for message: 0x{0:08x} of provider: '
          '{1:s}').format(message_identifier, lookup_key))
      return None

    return message_strings[0].string

  def _ReadWindowsEventLogProviders(self, storage_reader):
    """Reads the Windows EventLog providers.

    Args:
      storage_reader (StorageReader): storage reader.
    """
    # TODO: get windows eventlog providers to the source.
    self._windows_eventlog_providers = {}
    if storage_reader.HasAttributeContainers('windows_eventlog_provider'):
      for provider in storage_reader.GetAttributeContainers(
          'windows_eventlog_provider'):

        if provider.identifier:
          self._windows_eventlog_providers[provider.identifier] = provider

        for log_source in provider.log_sources:
          log_source = log_source.lower()
          self._windows_eventlog_providers[log_source] = provider

  def GetMessageString(
      self, provider_identifier, log_source, message_identifier, event_version):
    """Retrieves a specific Windows EventLog message string.

    Args:
      provider_identifier (str): EventLog provider identifier.
      log_source (str): EventLog source, such as "Application Error".
      message_identifier (int): message identifier.
      event_version (int): event version or None if not set.

    Returns:
      str: message string or None if not available.
    """
    message_string = self._GetCachedMessageString(
        provider_identifier, log_source, message_identifier, event_version)
    if not message_string:
      # TODO: change this logic.
      if self._storage_reader and self._storage_reader.HasAttributeContainers(
          'windows_eventlog_provider'):
        message_string = self._ReadWindowsEventLogMessageString(
            self._storage_reader, provider_identifier, log_source,
            message_identifier, event_version)
      else:
        message_string = self._GetWinevtRcDatabaseMessageString(
            log_source, message_identifier)

      if message_string:
        self._CacheMessageString(
            provider_identifier, log_source, message_identifier, event_version,
            message_string)

    return message_string



# -*- coding: utf-8 -*-
"""A plugin to generate a list of domains visited."""

from urllib import parse as urlparse

from plaso.analysis import interface
from plaso.analysis import manager


class UniqueDomainsVisitedPlugin(interface.AnalysisPlugin):
  """A plugin to generate a list all domains visited.

  This plugin will extract domains from browser history events extracted by
  Plaso. The list produced can be used to quickly determine if there has been
  a visit to a site of interest, for example, a known phishing site.
  """

  NAME = 'unique_domains_visited'

  _SUPPORTED_EVENT_DATA_TYPES = frozenset([
      'chrome:history:file_downloaded',
      'chrome:history:page_visited',
      'firefox:downloads:download',
      'firefox:places:page_visited',
      'macosx:lsquarantine',
      'msiecf:redirected',
      'msiecf:url',
      'msie:webcache:container',
      'opera:history',
      'safari:history:visit'])

  # pylint: disable=unused-argument
  def ExamineEvent(
      self, analysis_mediator, event, event_data, event_data_stream):
    """Analyzes an event and extracts domains from it.

    We only evaluate straightforward web history events, not visits which can
    be inferred by TypedURLs, cookies or other means.

    Args:
      analysis_mediator (AnalysisMediator): mediates interactions between
          analysis plugins and other components, such as storage and dfVFS.
      event (EventObject): event to examine.
      event_data (EventData): event data.
      event_data_stream (EventDataStream): event data stream.
    """
    if event_data.data_type not in self._SUPPORTED_EVENT_DATA_TYPES:
      return

    url = getattr(event_data, 'url', None)
    if url:
      parsed_url = urlparse.urlparse(url)
      domain = getattr(parsed_url, 'netloc', None)
      if domain:
        self._analysis_counter[domain] += 1


manager.AnalysisPluginManager.RegisterPlugin(UniqueDomainsVisitedPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the event data timeliner."""

import unittest

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.engine import timeliner
from plaso.storage.fake import writer as fake_writer

from tests import test_lib as shared_test_lib
from tests.engine import test_lib


class TestEventData1(events.EventData):
  """Test event data.

  Attributes:
    access_time (dfdatetime.DateTimeValues): access date and time.
    value (str): value.
  """

  DATA_TYPE = 'test:fs:stat'

  def __init__(self):
    """Initializes event data."""
    super(TestEventData1, self).__init__(data_type=self.DATA_TYPE)
    self.access_time = None
    self.value = None


class TestEventData2(events.EventData):
  """Test event data.

  Attributes:
    added_time (dfdatetime.DateTimeValues): added date and time.
    value (str): value.
  """

  DATA_TYPE = 'test:log:entry'

  def __init__(self):
    """Initializes event data."""
    super(TestEventData2, self).__init__(data_type=self.DATA_TYPE)
    self.access_time = None
    self.value = None


class EventDataTimelinerTest(test_lib.EngineTestCase):
  """Tests for the event data timeliner."""

  # pylint: disable=protected-access

  # pylint: disable=arguments-differ
  def _CreateStorageWriter(self, event_data, base_year=None):
    """Creates a storage writer object.

    Args:
      event_data (EventData): event data.
      base_year (Optional[int]): base year.

    Returns:
      FakeStorageWriter: storage writer.
    """
    storage_writer = fake_writer.FakeStorageWriter()
    storage_writer.Open()

    event_data_stream = events.EventDataStream()
    storage_writer.AddAttributeContainer(event_data_stream)

    event_data_stream_identifier = event_data_stream.GetIdentifier()

    if base_year:
      year_less_log_helper = events.YearLessLogHelper()
      year_less_log_helper.earliest_year = base_year
      year_less_log_helper.last_relative_year = 0

      year_less_log_helper.SetEventDataStreamIdentifier(
          event_data_stream_identifier)
      storage_writer.AddAttributeContainer(year_less_log_helper)

    event_data.SetEventDataStreamIdentifier(event_data_stream_identifier)
    storage_writer.AddAttributeContainer(event_data)

    return storage_writer

  def testGetBaseYear(self):
    """Tests the _GetBaseYear function."""
    event_data_timeliner = timeliner.EventDataTimeliner(
        data_location=shared_test_lib.TEST_DATA_PATH)

    current_year = event_data_timeliner._GetCurrentYear()

    event_data = TestEventData1()
    event_data.value = 'MyValue'

    # Test with year-less log helper.
    storage_writer = self._CreateStorageWriter(event_data, base_year=2012)

    # Ensure to reset the timeliner base years cache.
    event_data_timeliner._base_years = {}

    base_year = event_data_timeliner._GetBaseYear(storage_writer, event_data)
    self.assertEqual(base_year, 2012)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'timelining_warning')
    self.assertEqual(number_of_warnings, 0)

    # Test missing year-less log helper.
    storage_writer = self._CreateStorageWriter(event_data)

    # Ensure to reset the timeliner base years cache.
    event_data_timeliner._base_years = {}

    base_year = event_data_timeliner._GetBaseYear(storage_writer, event_data)
    self.assertEqual(base_year, current_year)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'timelining_warning')
    self.assertEqual(number_of_warnings, 1)

    # TODO: improve test coverage.

  def testGetCurrentYear(self):
    """Tests the _GetCurrentYear function."""
    event_data_timeliner = timeliner.EventDataTimeliner(
        data_location=shared_test_lib.TEST_DATA_PATH)

    current_year = event_data_timeliner._GetCurrentYear()
    self.assertIsNotNone(current_year)

  def testGetEvent(self):
    """Tests the _GetEvent function."""
    event_data_timeliner = timeliner.EventDataTimeliner(
        data_location=shared_test_lib.TEST_DATA_PATH)

    event_data = TestEventData1()
    event_data.value = 'MyValue'

    # Test a regular date and time value.
    storage_writer = self._CreateStorageWriter(event_data)

    date_time = dfdatetime_time_elements.TimeElementsInMicroseconds(
        time_elements_tuple=(2010, 8, 12, 20, 6, 31, 429876))

    # Ensure to reset the timeliner base years cache.
    event_data_timeliner._base_years = {}

    event = event_data_timeliner._GetEvent(
        storage_writer, event_data, None, date_time, 'Test Time')
    self.assertIsNotNone(event)
    self.assertIsNotNone(event.date_time)
    self.assertEqual(event.date_time.year, 2010)
    self.assertEqual(event.timestamp, 1281643591429876)

    date_time = dfdatetime_time_elements.TimeElementsInMicroseconds(
        time_elements_tuple=(2010, 8, 12, 20, 6, 31, 429876))

    # Test date time delta of February 29 with leap year.
    storage_writer = self._CreateStorageWriter(event_data, base_year=2012)

    date_time = dfdatetime_time_elements.TimeElementsInMicroseconds(
        is_delta=True, time_elements_tuple=(0, 2, 29, 20, 6, 31, 429876))

    # Ensure to reset the timeliner base years cache.
    event_data_timeliner._base_years = {}

    event = event_data_timeliner._GetEvent(
        storage_writer, event_data, None, date_time, 'Test Time')
    self.assertIsNotNone(event)
    self.assertIsNotNone(event.date_time)
    self.assertEqual(event.date_time.year, 2012)
    self.assertEqual(event.timestamp, 1330545991429876)

    date_time = dfdatetime_time_elements.TimeElementsInMicroseconds(
        is_delta=True, time_elements_tuple=(1, 8, 12, 20, 6, 31, 429876))

    # Test date time delta of February 29 with non-leap year.
    storage_writer = self._CreateStorageWriter(event_data, base_year=2013)

    date_time = dfdatetime_time_elements.TimeElementsInMicroseconds(
        is_delta=True, time_elements_tuple=(0, 2, 29, 20, 6, 31, 429876))

    # Ensure to reset the timeliner base years cache.
    event_data_timeliner._base_years = {}

    with self.assertRaises(ValueError):
      event_data_timeliner._GetEvent(
          storage_writer, event_data, None, date_time, 'Test Time')

    # Test date time delta without a base year.
    storage_writer = self._CreateStorageWriter(event_data)

    date_time = dfdatetime_time_elements.TimeElementsInMicroseconds(
        time_elements_tuple=(4, 2, 29, 20, 6, 31, 429876))

    # Ensure to reset the timeliner base years cache.
    event_data_timeliner._base_years = {}

    event = event_data_timeliner._GetEvent(
        storage_writer, event_data, None, date_time, 'Test Time')
    self.assertIsNotNone(event)
    self.assertIsNotNone(event.date_time)
    self.assertEqual(event.date_time.year, 4)
    self.assertEqual(event.timestamp, -62035818808570124)

  # TODO: add tests for _ProduceTimeliningWarning
  # TODO: add tests for _ReadConfigurationFile

  def testProcessEventData(self):
    """Tests the ProcessEventData function."""
    # Test creating an event.
    event_data_timeliner = timeliner.EventDataTimeliner(
        data_location=shared_test_lib.TEST_DATA_PATH)

    event_data = TestEventData1()
    event_data.access_time = (
        dfdatetime_time_elements.TimeElementsInMicroseconds(
            time_elements_tuple=(2010, 8, 12, 20, 6, 31, 429876)))
    event_data.value = 'MyValue'

    self.assertEqual(event_data_timeliner.number_of_produced_events, 0)

    storage_writer = self._CreateStorageWriter(event_data)

    event_data_timeliner.ProcessEventData(storage_writer, event_data, None)

    self.assertEqual(event_data_timeliner.number_of_produced_events, 1)

    # Test creating a placeholder event.
    event_data_timeliner = timeliner.EventDataTimeliner(
        data_location=shared_test_lib.TEST_DATA_PATH)

    event_data = TestEventData1()
    event_data.value = 'MyValue'

    self.assertEqual(event_data_timeliner.number_of_produced_events, 0)

    storage_writer = self._CreateStorageWriter(event_data)

    event_data_timeliner.ProcessEventData(storage_writer, event_data, None)

    self.assertEqual(event_data_timeliner.number_of_produced_events, 1)

    # Test creating no placeholder event.
    event_data_timeliner = timeliner.EventDataTimeliner(
        data_location=shared_test_lib.TEST_DATA_PATH)

    event_data = TestEventData2()
    event_data.value = 'MyValue'

    self.assertEqual(event_data_timeliner.number_of_produced_events, 0)

    storage_writer = self._CreateStorageWriter(event_data)

    event_data_timeliner.ProcessEventData(storage_writer, event_data, None)

    self.assertEqual(event_data_timeliner.number_of_produced_events, 0)

  def testSetPreferredTimeZone(self):
    """Tests the SetPreferredTimeZone function."""
    event_data_timeliner = timeliner.EventDataTimeliner(
        data_location=shared_test_lib.DATA_PATH)

    event_data_timeliner.SetPreferredTimeZone('Europe/Amsterdam')

    self.assertEqual(
        event_data_timeliner._preferred_time_zone.zone, 'Europe/Amsterdam')

    event_data_timeliner.SetPreferredTimeZone(None)

    self.assertIsNone(event_data_timeliner._preferred_time_zone)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The timeliner, which is used to generate events from event data."""

import collections
import copy
import datetime
import os
import pytz

from dfdatetime import interface as dfdatetime_interface
from dfdatetime import semantic_time as dfdatetime_semantic_time

from plaso.containers import events
from plaso.containers import warnings
from plaso.engine import yaml_timeliner_file
from plaso.lib import definitions


class EventDataTimeliner(object):
  """The event data timeliner.

  Attributes:
    number_of_produced_events (int): number of produced events.
    parsers_counter (collections.Counter): number of events per parser or
        parser plugin.
  """

  _DEFAULT_TIME_ZONE = pytz.UTC

  _INT64_MIN = -1 << 63
  _INT64_MAX = (1 << 63) - 1

  _TIMELINER_CONFIGURATION_FILENAME = 'timeliner.yaml'

  def __init__(
      self, data_location=None, preferred_year=None,
      system_configurations=None):
    """Initializes an event data timeliner.

    Args:
      data_location (Optional[str]): path of the timeliner configuration file.
      preferred_year (Optional[int]): preferred initial year value for year-less
          date and time values.
      system_configurations (Optional[list[SystemConfigurationArtifact]]):
          system configurations.
    """
    super(EventDataTimeliner, self).__init__()
    self._attribute_mappings = {}
    self._base_years = {}
    self._current_year = self._GetCurrentYear()
    self._data_location = data_location
    self._place_holder_event = set()
    self._preferred_time_zone = None
    self._preferred_year = preferred_year
    self._time_zone_per_path_spec = None

    self.number_of_produced_events = 0
    self.parsers_counter = collections.Counter()

    self._CreateTimeZonePerPathSpec(system_configurations)
    self._ReadConfigurationFile()

  def _CreateTimeZonePerPathSpec(self, system_configurations):
    """Creates the time zone per path specification lookup table.

    Args:
      system_configurations (list[SystemConfigurationArtifact]): system
          configurations.
    """
    self._time_zone_per_path_spec = {}
    for system_configuration in system_configurations or []:
      if system_configuration.time_zone:
        for path_spec in system_configuration.path_specs:
          if path_spec.parent:
            self._time_zone_per_path_spec[path_spec.parent] = (
                system_configuration.time_zone)

  def _GetBaseYear(self, storage_writer, event_data):
    """Retrieves the base year.

    Args:
      storage_writer (StorageWriter): storage writer.
      event_data (EventData): event data.

    Returns:
      int: base year.
    """
    # If preferred year is set considered it a user override, otherwise try
    # to determine the year based on the year-less log helper or fallback to
    # the current year.

    if self._preferred_year:
      return self._preferred_year

    event_data_stream_identifier = event_data.GetEventDataStreamIdentifier()
    if not event_data_stream_identifier:
      return self._current_year

    lookup_key = event_data_stream_identifier.CopyToString()

    base_year = self._base_years.get(lookup_key, None)
    if base_year:
      return base_year

    filter_expression = '_event_data_stream_identifier == "{0:s}"'.format(
        lookup_key)
    year_less_log_helpers = list(storage_writer.GetAttributeContainers(
        events.YearLessLogHelper.CONTAINER_TYPE,
        filter_expression=filter_expression))
    if not year_less_log_helpers:
      message = (
          'missing year-less log helper, defaulting to current year: '
          '{0:d}').format(self._current_year)
      self._ProduceTimeliningWarning(storage_writer, event_data, message)

      base_year = self._current_year

    else:
      earliest_year = year_less_log_helpers[0].earliest_year
      last_relative_year = year_less_log_helpers[0].last_relative_year
      latest_year = year_less_log_helpers[0].latest_year

      if earliest_year is None and latest_year is None:
        message = (
            'missing earliest and latest year in year-less log helper, '
            'defaulting to current year: {0:d}').format(self._current_year)
        self._ProduceTimeliningWarning(storage_writer, event_data, message)

        base_year = self._current_year

      elif earliest_year + last_relative_year < self._current_year:
        base_year = earliest_year

      elif latest_year < self._current_year:
        message = (
            'earliest year: {0:d} as base year would exceed current year: '
            '{1:d} + {2:d}, using latest year: {3:d}').format(
                earliest_year, self._current_year, last_relative_year,
                latest_year)
        self._ProduceTimeliningWarning(storage_writer, event_data, message)

        base_year = latest_year - last_relative_year

      else:
        message = (
            'earliest year: {0:d} and latest: year: {1:d} as base year '
            'would exceed current year: {2:d} + {3:d}, using current '
            'year').format(
                earliest_year, latest_year, self._current_year,
                last_relative_year)
        self._ProduceTimeliningWarning(storage_writer, event_data, message)

        base_year = self._current_year - last_relative_year

    self._base_years[lookup_key] = base_year

    return base_year

  def _GetCurrentYear(self):
    """Retrieves current year.

    Returns:
      int: the current year.
    """
    datetime_object = datetime.datetime.now()
    return datetime_object.year

  def _GetEvent(
      self, storage_writer, event_data, event_data_stream, date_time,
      date_time_description):
    """Retrieves an event.

    Args:
      storage_writer (StorageWriter): storage writer.
      event_data (EventData): event data.
      event_data_stream (EventDataStream): event data stream.
      date_time (dfdatetime.DateTimeValues): date and time values.
      date_time_description (str): description of the meaning of the date and
          time values.

    Returns:
      EventObject: event.
    """
    if date_time.is_delta:
      base_year = self._GetBaseYear(storage_writer, event_data)
      date_time = date_time.NewFromDeltaAndYear(base_year)

    timestamp = date_time.GetPlasoTimestamp()
    if timestamp is None:
      self._ProduceTimeliningWarning(
          storage_writer, event_data, 'unable to determine timestamp')

      date_time = dfdatetime_semantic_time.InvalidTime()
      timestamp = 0

    # Check for out of bounds timestamps, for example if a data format has
    # changed and the conversion leads to an incorrect large integer value.
    # Integer values that are larger than 64-bit will cause an OverflowError
    # in the SQLite storage.

    elif timestamp < self._INT64_MIN or timestamp > self._INT64_MAX:
      self._ProduceTimeliningWarning(
          storage_writer, event_data, 'timestamp out of bounds')

      date_time = dfdatetime_semantic_time.InvalidTime()
      timestamp = 0

    else:
      if date_time.is_local_time:
        time_zone = None
        if date_time.time_zone_hint:
          # TODO: cache time zones per hint.
          try:
            time_zone = pytz.timezone(date_time.time_zone_hint)
          except pytz.UnknownTimeZoneError:
            message = (
                'unsupported time zone hint: {0:s}, using default time '
                'zone').format(date_time.time_zone_hint)
            self._ProduceTimeliningWarning(storage_writer, event_data, message)

        if not time_zone and event_data_stream:
          try:
            time_zone = self._GetTimeZoneByPathSpec(event_data_stream.path_spec)
          except pytz.UnknownTimeZoneError:
            message = (
                'unsupported system time zone: {0:s}, using default time '
                'zone').format(date_time.time_zone_hint)
            self._ProduceTimeliningWarning(storage_writer, event_data, message)

        if not time_zone:
          time_zone = self._preferred_time_zone or self._DEFAULT_TIME_ZONE

        date_time = copy.deepcopy(date_time)
        date_time.is_local_time = False

        if time_zone != pytz.UTC:
          datetime_object = datetime.datetime(
              1970, 1, 1, 0, 0, 0, 0, tzinfo=None)
          datetime_object += datetime.timedelta(microseconds=timestamp)

          datetime_delta = time_zone.utcoffset(datetime_object, is_dst=False)
          seconds_delta = int(datetime_delta.total_seconds())
          timestamp -= seconds_delta * definitions.MICROSECONDS_PER_SECOND

          date_time.time_zone_offset = seconds_delta // 60

    event = events.EventObject()
    event.date_time = date_time
    event.timestamp = timestamp
    event.timestamp_desc = date_time_description

    event_data_identifier = event_data.GetIdentifier()
    event.SetEventDataIdentifier(event_data_identifier)

    return event

  def _GetTimeZoneByPathSpec(self, path_spec):
    """Retrieves a time zone for a specific path specification.

    Args:
      path_spec (dfvfs.PathSpec): path specification.

    Returns:
      pytz.tzfile: time zone or None if not available.

    Raises:
      pytz.UnknownTimeZoneError: if the time zone is unknown.
    """
    if not path_spec or not path_spec.parent:
      return None

    time_zone = self._time_zone_per_path_spec.get(path_spec.parent, None)
    if not time_zone:
      return None

    if isinstance(time_zone, str):
      try:
        time_zone = pytz.timezone(time_zone)
        self._time_zone_per_path_spec[path_spec.parent] = time_zone
      except pytz.UnknownTimeZoneError as exeception:
        self._time_zone_per_path_spec[path_spec.parent] = None
        raise exeception

    return time_zone

  def _ProduceTimeliningWarning(self, storage_writer, event_data, message):
    """Produces a timelining warning.

    Args:
      storage_writer (StorageWriter): storage writer.
      event_data (EventData): event data.
      message (str): message of the warning.
    """
    parser_chain = getattr(event_data, '_parser_chain', None)
    path_spec = None

    event_data_stream_identifier = event_data.GetEventDataStreamIdentifier()
    if event_data_stream_identifier:
      event_data_stream = storage_writer.GetAttributeContainerByIdentifier(
          events.EventDataStream.CONTAINER_TYPE, event_data_stream_identifier)
      if event_data_stream:
        path_spec = event_data_stream.path_spec

    warning = warnings.TimeliningWarning(
        message=message, parser_chain=parser_chain, path_spec=path_spec)
    storage_writer.AddAttributeContainer(warning)

  def _ReadConfigurationFile(self):
    """Reads a timeliner configuration file.

    Raises:
      KeyError: if the attribute mappings are already set for the corresponding
          data type.
    """
    path = os.path.join(
        self._data_location, self._TIMELINER_CONFIGURATION_FILENAME)

    configuration_file = yaml_timeliner_file.YAMLTimelinerConfigurationFile()
    for timeliner_definition in configuration_file.ReadFromFile(path):
      if timeliner_definition.data_type in self._attribute_mappings:
        raise KeyError(
            'Attribute mappings for data type: {0:s} already set.'.format(
                timeliner_definition.data_type))

      self._attribute_mappings[timeliner_definition.data_type] = (
          timeliner_definition.attribute_mappings)

      if timeliner_definition.place_holder_event:
        self._place_holder_event.add(timeliner_definition.data_type)

  def ProcessEventData(self, storage_writer, event_data, event_data_stream):
    """Generate events from event data.

    Args:
      storage_writer (StorageWriter): storage writer.
      event_data (EventData): event data.
      event_data_stream (EventDataStream): event data stream.
    """
    self.number_of_produced_events = 0

    attribute_mappings = self._attribute_mappings.get(
        event_data.data_type) or {}
    if (not attribute_mappings and
        event_data.data_type not in self._place_holder_event):
      return

    parser_name = None
    parser_chain = getattr(event_data, '_parser_chain', None)
    if parser_chain:
      parser_name = parser_chain.rsplit('/', maxsplit=1)[-1]

    number_of_events = 0
    for attribute_name, time_description in attribute_mappings.items():
      attribute_values = getattr(event_data, attribute_name, None) or []
      if not isinstance(attribute_values, list):
        attribute_values = [attribute_values]

      for attribute_value in attribute_values:
        if not isinstance(attribute_value, dfdatetime_interface.DateTimeValues):
          message = 'unsupported date time attribute: {0:s}'.format(
              attribute_name)
          self._ProduceTimeliningWarning(storage_writer, event_data, message)
          continue

        event = self._GetEvent(
            storage_writer, event_data, event_data_stream, attribute_value,
            time_description)

        try:
          storage_writer.AddAttributeContainer(event)
        except OverflowError as exception:
          message = 'unable to add event with error: {0!s}'.format(exception)
          self._ProduceTimeliningWarning(storage_writer, event_data, message)
          continue

        number_of_events += 1

        if parser_name:
          self.parsers_counter[parser_name] += 1
        self.parsers_counter['total'] += 1

        self.number_of_produced_events += 1

    # Create a place holder event for event_data without date and time
    # values to map.
    if (not number_of_events and
        event_data.data_type in self._place_holder_event):
      date_time = dfdatetime_semantic_time.NotSet()
      event = self._GetEvent(
          storage_writer, event_data, event_data_stream, date_time,
          definitions.TIME_DESCRIPTION_NOT_A_TIME)

      storage_writer.AddAttributeContainer(event)

      if parser_name:
        self.parsers_counter[parser_name] += 1
      self.parsers_counter['total'] += 1

      self.number_of_produced_events += 1

  def SetPreferredTimeZone(self, time_zone_string):
    """Sets the preferred time zone for zone-less date and time values.

    Args:
      time_zone_string (str): time zone such as "Europe/Amsterdam" or None if
          the time zone determined by preprocessing or the default should be
          used.

    Raises:
      ValueError: if the time zone is not supported.
    """
    time_zone = None
    if time_zone_string:
      try:
        time_zone = pytz.timezone(time_zone_string)
      except pytz.UnknownTimeZoneError:
        raise ValueError('Unsupported time zone: {0!s}'.format(
            time_zone_string))

    self._preferred_time_zone = time_zone



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the McAfee AV Log parser."""

import unittest

from plaso.parsers import mcafeeav

from tests.parsers import test_lib


class McafeeAccessProtectionUnitTest(test_lib.ParserTestCase):
  """Tests for the McAfee AV Log parser."""

  def testParse(self):
    """Tests the Parse function."""
    parser = mcafeeav.McafeeAccessProtectionParser()
    storage_writer = self._ParseFile(['AccessProtectionLog.txt'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 14)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # TODO: Test that the UTF-8 byte order mark gets removed from
    # the first line.

    # Test this entry:
    # 9/27/2013 2:42:26 PM  Blocked by Access Protection rule
    #   SOMEDOMAIN\someUser C:\Windows\System32\procexp64.exe C:\Program Files
    # (x86)\McAfee\Common Framework\UdaterUI.exe  Common Standard
    # Protection:Prevent termination of McAfee processes  Action blocked :
    # Terminate

    expected_event_values = {
        'action': 'Action blocked : Terminate',
        'data_type': 'av:mcafee:accessprotectionlog',
        'filename': 'C:\\Windows\\System32\\procexp64.exe',
        'rule': (
            'Common Standard Protection:Prevent termination of McAfee '
            'processes'),
        'status': 'Blocked by Access Protection rule',
        'trigger_location': (
            'C:\\Program Files (x86)\\McAfee\\Common Framework\\Frame'
            'workService.exe'),
        'username': 'SOMEDOMAIN\\someUser',
        'written_time': '2013-09-27T14:42:39'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Spotlight searched terms plist plugin."""

import unittest

from plaso.parsers.plist_plugins import spotlight_searched_terms

from tests.parsers.plist_plugins import test_lib


class SpotlightSearchedTermsPlistPluginTest(test_lib.PlistPluginTestCase):
  """Tests for the Spotlight searched terms plist plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plist_name = 'com.apple.spotlight.plist'

    plugin = spotlight_searched_terms.SpotlightSearchedTermsPlistPlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 9)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'spotlight_searched_terms:entry',
        'display_name': 'Grab',
        'last_used_time': '2013-12-23T18:21:41.900938+00:00',
        'path': '/Applications/Utilities/Grab.app',
        'search_term': 'gr'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 8)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the MacOS local users plist plugin."""

import unittest

from plaso.parsers.plist_plugins import macos_user

from tests.parsers.plist_plugins import test_lib


class MacOSUserPlistPluginTest(test_lib.PlistPluginTestCase):
  """Tests for the MacOS local user plist plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plist_name = 'user.plist'

    plugin = macos_user.MacOSUserPlistPlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'macos:user:entry',
        'fullname': 'Joaquin Moreno',
        'last_login_attempt_time': None,
        'last_login_time': None,
        'last_password_set_time': '2013-12-28T04:35:47+00:00',
        'password_hash': (
            '$ml$37313$fa6cac1869263baa85cffc5e77a3d4ee164b7'
            '5536cae26ce8547108f60e3f554$a731dbb0e386b169af8'
            '9fbb33c255ceafc083c6bc5194853f72f11c550c42e4625'
            'ef113b66f3f8b51fc3cd39106bad5067db3f7f1491758ff'
            'e0d819a1b0aba20646fd61345d98c0c9a411bfd1144dd4b'
            '3c40ec0f148b66d5b9ab014449f9b2e103928ef21db6e25'
            'b536a60ff17a84e985be3aa7ba3a4c16b34e0d1d2066ae178'),
        'user_identifier': '501',
        'username': 'user'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The year-less log format helper mix-in."""

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.resolver import resolver as path_spec_resolver

from plaso.containers import events


class YearLessLogFormatHelper(object):
  """Year-less log format helper mix-in."""

  _MONTH_DICT = {
      'jan': 1,
      'feb': 2,
      'mar': 3,
      'apr': 4,
      'may': 5,
      'jun': 6,
      'jul': 7,
      'aug': 8,
      'sep': 9,
      'oct': 10,
      'nov': 11,
      'dec': 12}

  _VALID_MONTHS = frozenset(range(1, 13))

  def __init__(self):
    """Initializes the year-less log format helper mix-in."""
    super(YearLessLogFormatHelper, self).__init__()
    self._base_year = None
    self._maximum_year = None
    self._month = None
    self._relative_year = 0
    self._year = 0

  def _GetYearsFromFileEntry(self, file_entry):
    """Retrieves the years from the file entry date and time values.

    Args:
      file_entry (dfvfs.FileEntry): file entry.

    Returns:
      set[int]: years of the file entry.
    """
    years = set()

    for attribute_name in ('change_time', 'creation_time', 'modification_time'):
      date_time = getattr(file_entry, attribute_name, None)
      if date_time:
        year, _, _ = date_time.GetDate()
        years.add(year)

    return years

  def _GetMonthFromString(self, month_string):
    """Retrieves a numeric month value from a string.

    Args:
      month_string (str): month formatted as a string.

    Returns:
      int: month formatted as an integer, where January is 1.
    """
    # TODO: add support for localization.
    return self._MONTH_DICT.get(month_string.lower(), None)

  def _GetRelativeYear(self):
    """Retrieves the relative year.

    Returns:
      int: relative year.
    """
    return self._relative_year

  def _GetYear(self):
    """Retrieves the year.

    Returns:
      int: year.
    """
    return self._year

  def _SetEstimatedYear(self, parser_mediator):
    """Sets the year based on the parser mediator year estimation.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
    """
    self._base_year = None
    self._maximum_year = None
    self._month = None
    self._relative_year = 0
    self._year = 0

    years = set()

    file_entry = parser_mediator.GetFileEntry()
    if file_entry:
      years = self._GetYearsFromFileEntry(file_entry)

    if not years and file_entry.type_indicator in (
        dfvfs_definitions.TYPE_INDICATOR_COMPRESSED_STREAM,
        dfvfs_definitions.TYPE_INDICATOR_GZIP):

      parent_file_entry = path_spec_resolver.Resolver.OpenFileEntry(
          file_entry.path_spec.parent,
          resolver_context=parser_mediator.resolver_context)
      if parent_file_entry:
        years = self._GetYearsFromFileEntry(parent_file_entry)

    if years:
      self._base_year = min(years)
      self._maximum_year = max(years)
      self._year = self._base_year

  def _SetMonthAndYear(self, month, year):
    """Sets the month and year.

    Args:
      month (int): month.
      year (int): year.

    Raises:
      ValueError: if month contains an unsupported value.
    """
    if month not in self._VALID_MONTHS:
      raise ValueError('Invalid month: {0!s}'.format(month))

    self._month = month
    self._relative_year = 0
    self._year = year

  def _UpdateYear(self, month):
    """Updates the year based on the month observed in the log format.

    Args:
      month (int): month observed in the log format, where January is 1.

    Raises:
      ValueError: if month contains an unsupported value.
    """
    if month not in self._VALID_MONTHS:
      raise ValueError('Invalid month: {0!s}'.format(month))

    if self._month:
      # Account for log formats that allow out-of-order date and time values
      # (Apr->May->Apr) such as rsyslog with the RepeatedMsgReduction setting
      # enabled.
      if (month + 1) < self._month:
        self._relative_year += 1
        self._year += 1

      # Account for out-of-order Jan->Dec->Jan with the exception of the start
      # of the log file.
      elif self._relative_year > 0 and self._month == 1 and month == 12:
        self._relative_year -= 1
        self._year -= 1

    self._month = month

  def GetYearLessLogHelper(self):
    """Retrieves a year-less log helper attribute container.

    Returns:
      YearLessLogHelper: year-less log helper.
    """
    year_less_log_helper = events.YearLessLogHelper()
    year_less_log_helper.earliest_year = self._base_year
    year_less_log_helper.last_relative_year = self._relative_year
    year_less_log_helper.latest_year = self._maximum_year

    return year_less_log_helper



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the psteal CLI tool."""

import os
import unittest

from dfvfs.resolver import resolver as dfvfs_resolver

from plaso.cli import psteal_tool
from plaso.lib import errors

from tests import test_lib as shared_test_lib
from tests.cli import test_lib


class PstealToolTest(test_lib.CLIToolTestCase):
  """Tests for the psteal CLI tool."""

  # pylint: disable=protected-access

  _BDE_PASSWORD = 'bde-TEST'

  def _CheckOutput(self, output, expected_output):
    """Compares the output against the expected output.

    The actual processing time is ignored, since it can vary.

    Args:
      output (str): tool output.
      expected_output (list[str]): expected tool output.
    """
    output = output.split('\n')

    self.assertEqual(output[:3], expected_output[:3])
    self.assertTrue(output[3].startswith('Processing time\t\t: '))
    self.assertEqual(output[4:], expected_output[4:])

  def testFailWhenOutputAlreadyExists(self):
    """Test to make sure the tool raises when the output file already exists."""
    test_artifacts_path = self._GetTestFilePath(['artifacts'])
    self._SkipIfPathNotExists(test_artifacts_path)

    test_file_path = self._GetTestFilePath(['psort_test.plaso'])
    self._SkipIfPathNotExists(test_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = test_artifacts_path
    options.source = 'unused_source'
    options.status_view_interval = 0.5
    options.status_view_mode = 'none'
    options.storage_file = test_file_path

    with shared_test_lib.TempDirectory() as temp_directory:
      options.log_file = os.path.join(temp_directory, 'output.log')
      options.write = os.path.join(temp_directory, 'output.txt')

      with open(options.write, 'w', encoding='utf-8') as file_object:
        file_object.write('bogus')

      # Test when output file already exists.
      # Escape \ otherwise assertRaisesRegexp can error with:
      # error: bogus escape: '\\1'
      expected_error = 'Output file already exists: {0:s}.'.format(
          options.write.replace('\\', '\\\\'))
      with self.assertRaisesRegex(errors.BadConfigOption, expected_error):
        test_tool.ParseOptions(options)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    test_artifacts_path = self._GetTestFilePath(['artifacts'])
    self._SkipIfPathNotExists(test_artifacts_path)

    test_file_path = self._GetTestFilePath(['testdir'])
    self._SkipIfPathNotExists(test_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    # Test when the output file is missing.
    options = test_lib.TestOptions()
    options.artifact_definitions_path = test_artifacts_path
    options.source = 'source'
    options.status_view_interval = 0.5
    options.status_view_mode = 'none'

    expected_error = 'Output format: dynamic requires an output file'
    with self.assertRaisesRegex(errors.BadConfigOption, expected_error):
      test_tool.ParseOptions(options)

    # Test when the source is missing.
    options = test_lib.TestOptions()
    options.status_view_interval = 0.5
    options.status_view_mode = 'none'
    options.write = 'output.csv'

    expected_error = 'Missing source path.'
    with self.assertRaisesRegex(errors.BadConfigOption, expected_error):
      test_tool.ParseOptions(options)

    # Test when both source and output are specified.
    with shared_test_lib.TempDirectory() as temp_directory:
      options.log_file = os.path.join(temp_directory, 'output.log')
      options.source = test_file_path
      options.write = os.path.join(temp_directory, 'dynamic.out')

      test_tool.ParseOptions(options)

      with open(options.write, 'w', encoding='utf-8') as file_object:
        file_object.write('bogus')

      # Test when output file already exists.
      # Escape \ otherwise assertRaisesRegexp can error with:
      # error: bogus escape: '\\1'
      expected_error = 'Output file already exists: {0:s}.'.format(
          options.write.replace('\\', '\\\\'))
      with self.assertRaisesRegex(errors.BadConfigOption, expected_error):
        test_tool.ParseOptions(options)

  def testParseArguments(self):
    """Tests the ParseArguments function"""
    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    # Test ParseArguments with no output file nor source.
    result = test_tool.ParseArguments([])
    self.assertFalse(result)
    output = output_writer.ReadOutput()
    expected_error = 'ERROR: Missing source path.'
    self.assertIn(expected_error, output)

  def testExtractEventsFromSourceDirectory(self):
    """Tests the ExtractEventsFromSources function on a directory."""
    test_artifacts_path = self._GetTestFilePath(['artifacts'])
    self._SkipIfPathNotExists(test_artifacts_path)

    test_file_path = self._GetTestFilePath(['testdir'])
    self._SkipIfPathNotExists(test_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = test_artifacts_path
    options.quiet = True
    options.status_view_interval = 0.5
    options.status_view_mode = 'none'
    options.source = test_file_path

    with shared_test_lib.TempDirectory() as temp_directory:
      options.log_file = os.path.join(temp_directory, 'output.log')
      options.storage_file = os.path.join(temp_directory, 'storage.plaso')
      options.write = os.path.join(temp_directory, 'output.txt')

      test_tool.ParseOptions(options)

      test_tool.ExtractEventsFromSources()

      expected_output = [
          '',
          'Source path\t\t: {0:s}'.format(options.source),
          'Source type\t\t: directory',
          'Processing time\t\t: 00:00:00',
          '',
          'Processing started.',
          'Processing completed.',
          '',
          '']

      output = output_writer.ReadOutput()
      self._CheckOutput(output, expected_output)

  def testExtractEventsFromSourceBDEImage(self):
    """Tests the ExtractEventsFromSources function on an image with BDE."""
    test_artifacts_path = self._GetTestFilePath(['artifacts'])
    self._SkipIfPathNotExists(test_artifacts_path)

    test_file_path = self._GetTestFilePath(['bdetogo.raw'])
    self._SkipIfPathNotExists(test_file_path)

    dfvfs_resolver.Resolver.key_chain.Empty()

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = test_artifacts_path
    options.credentials = ['password:{0:s}'.format(self._BDE_PASSWORD)]
    options.quiet = True
    options.status_view_interval = 0.5
    options.status_view_mode = 'none'
    options.source = test_file_path

    with shared_test_lib.TempDirectory() as temp_directory:
      options.log_file = os.path.join(temp_directory, 'output.log')
      options.storage_file = os.path.join(temp_directory, 'storage.plaso')
      options.write = os.path.join(temp_directory, 'output.txt')

      test_tool.ParseOptions(options)

      test_tool.ExtractEventsFromSources()

      expected_output = [
          '',
          'Source path\t\t: {0:s}'.format(options.source),
          'Source type\t\t: storage media image',
          'Processing time\t\t: 00:00:00',
          '',
          'Processing started.',
          'Processing completed.',
          '',
          '']

      output = output_writer.ReadOutput()
      self._CheckOutput(output, expected_output)

  def testExtractEventsFromSourcesImage(self):
    """Tests the ExtractEventsFromSources function on a single partition."""
    test_artifacts_path = self._GetTestFilePath(['artifacts'])
    self._SkipIfPathNotExists(test_artifacts_path)

    test_file_path = self._GetTestFilePath(['ímynd.dd'])
    self._SkipIfPathNotExists(test_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = test_artifacts_path
    options.quiet = True
    options.status_view_interval = 0.5
    options.status_view_mode = 'none'
    options.source = test_file_path

    with shared_test_lib.TempDirectory() as temp_directory:
      options.log_file = os.path.join(temp_directory, 'output.log')
      options.storage_file = os.path.join(temp_directory, 'storage.plaso')
      options.write = os.path.join(temp_directory, 'output.txt')

      test_tool.ParseOptions(options)

      test_tool.ExtractEventsFromSources()

      expected_output = [
          '',
          'Source path\t\t: {0:s}'.format(options.source),
          'Source type\t\t: storage media image',
          'Processing time\t\t: 00:00:00',
          '',
          'Processing started.',
          'Processing completed.',
          '',
          '']

      output = output_writer.ReadOutput()
      self._CheckOutput(output, expected_output)

  def testExtractEventsFromSourcePartitionedImage(self):
    """Tests the ExtractEventsFromSources function on a partitioned image."""
    test_artifacts_path = self._GetTestFilePath(['artifacts'])
    self._SkipIfPathNotExists(test_artifacts_path)

    # Note that the source file is a RAW (VMDK flat) image.
    test_file_path = self._GetTestFilePath(['multi_partition_image.vmdk'])
    self._SkipIfPathNotExists(test_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = test_artifacts_path
    options.partitions = 'all'
    options.quiet = True
    options.status_view_interval = 0.5
    options.status_view_mode = 'none'
    options.source = test_file_path

    with shared_test_lib.TempDirectory() as temp_directory:
      options.log_file = os.path.join(temp_directory, 'output.log')
      options.storage_file = os.path.join(temp_directory, 'storage.plaso')
      options.write = os.path.join(temp_directory, 'output.txt')

      test_tool.ParseOptions(options)

      test_tool.ExtractEventsFromSources()

      expected_output = [
          '',
          'Source path\t\t: {0:s}'.format(options.source),
          'Source type\t\t: storage media image',
          'Processing time\t\t: 00:00:00',
          '',
          'Processing started.',
          'Processing completed.',
          '',
          '']

      output = output_writer.ReadOutput()
      self._CheckOutput(output, expected_output)

  def testExtractEventsFromSourceVSSImage(self):
    """Tests the ExtractEventsFromSources function on an image with VSS."""
    test_artifacts_path = self._GetTestFilePath(['artifacts'])
    self._SkipIfPathNotExists(test_artifacts_path)

    test_file_path = self._GetTestFilePath(['vsstest.qcow2'])
    self._SkipIfPathNotExists(test_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = test_artifacts_path
    options.quiet = True
    options.single_process = True
    options.status_view_interval = 0.5
    options.status_view_mode = 'none'
    options.source = test_file_path
    options.unattended = True
    options.vss_stores = 'all'

    with shared_test_lib.TempDirectory() as temp_directory:
      options.log_file = os.path.join(temp_directory, 'output.log')
      options.storage_file = os.path.join(temp_directory, 'storage.plaso')
      options.write = os.path.join(temp_directory, 'output.txt')

      test_tool.ParseOptions(options)

      test_tool.ExtractEventsFromSources()

      expected_output = [
          '',
          'Source path\t\t: {0:s}'.format(options.source),
          'Source type\t\t: storage media image',
          'Processing time\t\t: 00:00:00',
          '',
          'Processing started.',
          'Processing completed.',
          '',
          'Number of warnings generated while extracting events: 3.',
          '',
          'Use pinfo to inspect warnings in more detail.',
          '',
          '']

      output = output_writer.ReadOutput()
      self._CheckOutput(output, expected_output)

  def testExtractEventsFromSourceSingleFile(self):
    """Tests the ExtractEventsFromSources function on a single file."""
    test_artifacts_path = self._GetTestFilePath(['artifacts'])
    self._SkipIfPathNotExists(test_artifacts_path)

    test_file_path = self._GetTestFilePath(['System.evtx'])
    self._SkipIfPathNotExists(test_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = test_artifacts_path
    options.quiet = True
    options.status_view_interval = 0.5
    options.status_view_mode = 'none'
    options.source = test_file_path

    with shared_test_lib.TempDirectory() as temp_directory:
      options.log_file = os.path.join(temp_directory, 'output.log')
      options.storage_file = os.path.join(temp_directory, 'storage.plaso')
      options.write = os.path.join(temp_directory, 'output.txt')

      test_tool.ParseOptions(options)

      test_tool.ExtractEventsFromSources()

      expected_output = [
          '',
          'Source path\t\t: {0:s}'.format(options.source),
          'Source type\t\t: single file',
          'Processing time\t\t: 00:00:00',
          '',
          'Processing started.',
          'Processing completed.',
          '',
          '']

      output = output_writer.ReadOutput()
      self._CheckOutput(output, expected_output)

  def testProcessStorage(self):
    """Test the ProcessStorage function"""
    test_artifacts_path = self._GetTestFilePath(['artifacts'])
    self._SkipIfPathNotExists(test_artifacts_path)

    test_file_path = self._GetTestFilePath(['psort_test.plaso'])
    self._SkipIfPathNotExists(test_file_path)

    expected_output_file_path = self._GetTestFilePath([
        'end_to_end', 'dynamic.log'])
    self._SkipIfPathNotExists(expected_output_file_path)

    output_writer = test_lib.TestOutputWriter(encoding='utf-8')
    test_tool = psteal_tool.PstealTool(output_writer=output_writer)

    options = test_lib.TestOptions()
    options.artifact_definitions_path = test_artifacts_path
    options.dynamic_time = True
    options.status_view_interval = 0.5
    options.storage_file = test_file_path
    options.source = 'unused_source'

    with shared_test_lib.TempDirectory() as temp_directory:
      options.log_file = os.path.join(temp_directory, 'output.log')
      options.write = os.path.join(temp_directory, 'output.txt')

      test_tool.ParseOptions(options)
      test_tool.ProcessStorage()

      with open(
          expected_output_file_path, 'rt', encoding='utf-8') as file_object:
        expected_output = file_object.read()

      with open(options.write, 'rt', encoding='utf-8') as file_object:
        result_output = file_object.read()

      result_output = sorted(result_output.split('\n'))
      expected_output = sorted(expected_output.split('\n'))
      self.assertEqual(result_output, expected_output)

    output = output_writer.ReadOutput()
    self.assertIn('Processing completed.', output)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Plist parser plugin for MacOS software update plist files."""

from plaso.containers import events
from plaso.parsers import plist
from plaso.parsers.plist_plugins import interface


class MacOSSoftwareUpdateEventData(events.EventData):
  """MacOS software update event data.

  Attributes:
    full_update_time (dfdatetime.DateTimeValues): date and time of last
        full MacOS software update.
    recommended_updates (list[str]): recommended updates.
    system_version (str): operating system version.
    update_time (dfdatetime.DateTimeValues): date and time of last
        MacOS software update.
  """

  DATA_TYPE = 'macos:software_updata:entry'

  def __init__(self):
    """Initializes event data."""
    super(MacOSSoftwareUpdateEventData, self).__init__(data_type=self.DATA_TYPE)
    self.full_update_time = None
    self.recommended_updates = None
    self.system_version = None
    self.update_time = None


class MacOSSoftwareUpdatePlistPlugin(interface.PlistPlugin):
  """Plist parser plugin for MacOS software update plist files.

  Further details about the extracted fields:
    LastFullSuccessfulDate:
      timestamp when MacOS was full update.
    LastSuccessfulDate:
      timestamp when MacOS was partially update.
  """

  NAME = 'macos_software_update'
  DATA_FORMAT = 'MacOS software update plist file'

  PLIST_PATH_FILTERS = frozenset([
      interface.PlistPathFilter('com.apple.SoftwareUpdate.plist')])

  PLIST_KEYS = frozenset([
      'LastFullSuccessfulDate', 'LastSuccessfulDate',
      'LastAttemptSystemVersion', 'LastUpdatesAvailable',
      'LastRecommendedUpdatesAvailable', 'RecommendedUpdates'])

  # pylint: disable=arguments-differ
  def _ParsePlist(self, parser_mediator, match=None, **unused_kwargs):
    """Extracts relevant MacOS update entries.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      match (Optional[dict[str: object]]): keys extracted from PLIST_KEYS.
    """
    last_full_successful_date = match.get('LastFullSuccessfulDate', None)
    last_successful_date = match.get('LastSuccessfulDate', None)

    recommended_updates = []
    if match.get('LastUpdatesAvailable', None):
      for update_property in match.get('RecommendedUpdates', []):
        identifier = update_property.get('Identifier', None)
        product_key = update_property.get('Product Key', None)

        recommended_updates.append(
            '{0:s} ({1:s})'.format(identifier, product_key))

    event_data = MacOSSoftwareUpdateEventData()
    event_data.full_update_time = self._GetDateTimeValueFromPlistKey(
         match, 'LastFullSuccessfulDate')
    event_data.recommended_updates = recommended_updates or None
    event_data.system_version = match.get('LastAttemptSystemVersion', None)

    # Only set update_time if it differs from full_update_time.
    if (last_successful_date and
        last_successful_date != last_full_successful_date):
      event_data.update_time = self._GetDateTimeValueFromPlistKey(
           match, 'LastSuccessfulDate')

    parser_mediator.ProduceEventData(event_data)


plist.PlistParser.RegisterPlugin(MacOSSoftwareUpdatePlistPlugin)



# -*- coding: utf-8 -*-
"""This file imports Python modules that register CLI helpers."""

from plaso.cli.helpers import analysis_plugins
from plaso.cli.helpers import archives
from plaso.cli.helpers import artifact_definitions
from plaso.cli.helpers import artifact_filters

try:
  from plaso.cli.helpers import bloom_analysis
except ImportError:
  pass

from plaso.cli.helpers import codepage
from plaso.cli.helpers import data_location
from plaso.cli.helpers import date_filters
from plaso.cli.helpers import dynamic_output
from plaso.cli.helpers import event_filters
from plaso.cli.helpers import extraction
from plaso.cli.helpers import filter_file
from plaso.cli.helpers import hashers
from plaso.cli.helpers import language
from plaso.cli.helpers import nsrlsvr_analysis
from plaso.cli.helpers import opensearch_output
from plaso.cli.helpers import opensearch_ts_output
from plaso.cli.helpers import output_modules
from plaso.cli.helpers import parsers
from plaso.cli.helpers import profiling
from plaso.cli.helpers import process_resources
from plaso.cli.helpers import sessionize_analysis
from plaso.cli.helpers import status_view
from plaso.cli.helpers import storage_format
from plaso.cli.helpers import tagging_analysis
from plaso.cli.helpers import temporary_directory
from plaso.cli.helpers import viper_analysis
from plaso.cli.helpers import virustotal_analysis
from plaso.cli.helpers import xlsx_output
from plaso.cli.helpers import yara_rules
from plaso.cli.helpers import vfs_backend
from plaso.cli.helpers import workers



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the iOS Kik messenger SQLite database plugin."""

import unittest

from plaso.parsers.sqlite_plugins import ios_kik

from tests.parsers.sqlite_plugins import test_lib


class IOSKikMessageTest(test_lib.SQLitePluginTestCase):
  """Tests for the iOS Kik messenger SQLite database plugin."""

  def testProcess(self):
    """Test the Process function."""
    plugin = ios_kik.IOSKikPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['kik_ios.sqlite'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 60)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'body': 'Hello',
        'data_type': 'ios:kik:messaging',
        'displayname': 'Ken Doh',
        'message_status': 94,
        'message_type': 2,
        'received_time': '2015-06-29T12:26:11.584833+00:00',
        'username': 'ken.doh'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""JSON-L parser plugin for Docker container log files."""

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.parsers import jsonl_parser
from plaso.parsers.jsonl_plugins import interface


class DockerContainerLogEventData(events.EventData):
  """Docker container log event data.

  Attributes:
    container_identifier (str): identifier of the container (SHA256).
    log_line (str): log line.
    log_source (str): log source.
    written_time (dfdatetime.DateTimeValues): date and time the entry was
        written.
  """

  DATA_TYPE = 'docker:container:log:entry'

  def __init__(self):
    """Initializes event data."""
    super(DockerContainerLogEventData, self).__init__(data_type=self.DATA_TYPE)
    self.container_identifier = None
    self.log_line = None
    self.log_source = None
    self.written_time = None


class DockerContainerLogJSONLPlugin(interface.JSONLPlugin):
  """JSON-L parser plugin for Docker container log files.

  This parser handles per Docker container log files stored in:
  DOCKER_DIR/containers/<container_identifier>/<container_identifier>-json.log
  """

  NAME = 'docker_container_log'
  DATA_FORMAT = 'Docker container log file'

  def __init__(self):
    """Initializes a JSON-L parser plugin."""
    super(DockerContainerLogJSONLPlugin, self).__init__()
    self._container_identifier = None

  def _GetContainerIdentifierFromPath(self, parser_mediator):
    """Extracts a container identifier from a path.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.

    Returns:
      str: container identifier.
    """
    file_entry = parser_mediator.GetFileEntry()
    file_system = file_entry.GetFileSystem()

    path_segments = file_system.SplitPath(file_entry.path_spec.location)
    # TODO: validate format of container identifier.
    return path_segments[-2]

  def _ParseRecord(self, parser_mediator, json_dict):
    """Parses a Docker container log record.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      json_dict (dict): JSON dictionary of the log record.
    """
    if not self._container_identifier:
      self._container_identifier = self._GetContainerIdentifierFromPath(
          parser_mediator)

    # TODO: escape special characters in log line.
    log_line = self._GetJSONValue(json_dict, 'log', default_value='')

    event_data = DockerContainerLogEventData()
    event_data.container_identifier = self._container_identifier
    event_data.log_line = log_line or None
    event_data.log_source = self._GetJSONValue(json_dict, 'stream')
    event_data.written_time = self._ParseISO8601DateTimeString(
        parser_mediator, json_dict, 'time')

    parser_mediator.ProduceEventData(event_data)

  def CheckRequiredFormat(self, json_dict):
    """Check if the log record has the minimal structure required by the plugin.

    Args:
      json_dict (dict): JSON dictionary of the log record.

    Returns:
      bool: True if this is the correct parser, False otherwise.
    """
    log = json_dict.get('log') or None
    stream = json_dict.get('stream') or None
    time = json_dict.get('time') or None

    if None in (log, stream, time):
      return False

    date_time = dfdatetime_time_elements.TimeElementsInMicroseconds()

    try:
      date_time.CopyFromStringISO8601(time)
    except ValueError:
      return False

    self._container_identifier = None

    return True


jsonl_parser.JSONLParser.RegisterPlugin(DockerContainerLogJSONLPlugin)



#!/usr/bin/env python3
# -*_ coding: utf-8 -*-
"""Tests for the ZSH extended history text parser plugin."""

import unittest

from dfvfs.file_io import fake_file_io
from dfvfs.path import fake_path_spec
from dfvfs.resolver import context as dfvfs_context

from plaso.parsers import text_parser
from plaso.parsers.text_plugins import zsh_extended_history

from tests.parsers.text_plugins import test_lib


class ZshExtendedHistoryTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for the ZSH extended history text parser plugin."""

  def testCheckRequiredFormat(self):
    """Tests for the CheckRequiredFormat method."""
    plugin = zsh_extended_history.ZshExtendedHistoryTextPlugin()

    resolver_context = dfvfs_context.Context()
    test_path_spec = fake_path_spec.FakePathSpec(location='/file.txt')

    file_object = fake_file_io.FakeFile(
        resolver_context, test_path_spec, b': 1457771210:0;cd plaso')
    file_object.Open()

    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(None, text_reader)
    self.assertTrue(result)

    file_object = fake_file_io.FakeFile(
        resolver_context, test_path_spec, b': 2016-03-26 11:54:53;0;cd plaso')
    file_object.Open()

    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(None, text_reader)
    self.assertFalse(result)

  def testProcess(self):
    """Tests the Process function."""
    plugin = zsh_extended_history.ZshExtendedHistoryTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['zsh_extended_history.txt'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 4)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'command': 'cd plaso',
        'data_type': 'shell:zsh:history',
        'elapsed_seconds': 0,
        'last_written_time': '2016-03-12T08:26:50+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Base class for a process that handles tasks used in multi-processing."""

import os

from plaso.lib import definitions
from plaso.multi_process import base_process
from plaso.storage import factory as storage_factory

try:
  # pylint: disable=ungrouped-imports
  import redis
  from plaso.storage.redis import redis_store
except ModuleNotFoundError:
  redis = None
  redis_store = None


class MultiProcessTaskProcess(base_process.MultiProcessBaseProcess):
  """Interface for multi-processing process that handles tasks."""

  # pylint: disable=abstract-method

  def __init__(
      self, processing_configuration, enable_sigsegv_handler=False, **kwargs):
    """Initializes a process.

    Args:
      processing_configuration (ProcessingConfiguration): processing
          configuration.
      enable_sigsegv_handler (Optional[bool]): True if the SIGSEGV handler
          should be enabled.
      kwargs (dict[str,object]): keyword arguments to pass to
          multiprocessing.Process.
    """
    processed_task_storage_path = None
    if processing_configuration.task_storage_path:
      processed_task_storage_path = os.path.join(
          processing_configuration.task_storage_path, 'processed')

    super(MultiProcessTaskProcess, self).__init__(
        processing_configuration, **kwargs)
    self._processed_task_storage_path = processed_task_storage_path
    self._storage_factory = storage_factory.StorageFactory
    self._task_storage_path = processing_configuration.task_storage_path

  def _FinalizeTaskStorageWriter(self, task_storage_format, task):
    """Finalizes a storage writer for a task.

    Args:
      task_storage_format (str): storage format used to store task results.
      task (Task): task the storage changes are part of.

    Raises:
      IOError: if the SQLite task storage file cannot be renamed.
      OSError: if the SQLite task storage file cannot be renamed.
    """
    if task.storage_format == definitions.STORAGE_FORMAT_REDIS and redis_store:
      url = redis_store.RedisStore.DEFAULT_REDIS_URL
      redis_client = redis.from_url(url=url, socket_timeout=60)
      redis_client.client_setname('task_process')

      redis_hash_name = self._GetProcessedRedisHashName(task.session_identifier)
      redis_client.hset(redis_hash_name, key=task.identifier, value=b'true')

    elif task.storage_format == definitions.STORAGE_FORMAT_SQLITE:
      storage_file_path = self._GetTaskStorageFilePath(
          task_storage_format, task)
      processed_storage_file_path = self._GetProcessedStorageFilePath(
          task_storage_format, task)

      try:
        os.rename(storage_file_path, processed_storage_file_path)
      except OSError as exception:
        raise IOError((
            'Unable to rename task storage file: {0:s} with error: '
            '{1!s}').format(storage_file_path, exception))

  def _GetProcessedRedisHashName(self, session_identifier):
    """Retrieves the Redis hash name of a processed task store.

    Args:
      session_identifier (str): the identifier of the session the tasks are
          part of.

    Returns:
      str: Redis hash name of a task store.
    """
    return '{0:s}-processed'.format(session_identifier)

  def _GetProcessedStorageFilePath(self, task_storage_format, task):
    """Retrieves the path of a task storage file in the processed directory.

    Args:
      task_storage_format (str): storage format used to store task results.
      task (Task): task the storage changes are part of.

    Returns:
      str: path of a task storage file in the processed directory or None if
          not est.
    """
    if task_storage_format == definitions.STORAGE_FORMAT_SQLITE:
      filename = '{0:s}.plaso'.format(task.identifier)
      return os.path.join(self._processed_task_storage_path, filename)

    return None

  def _GetTaskStorageFilePath(self, task_storage_format, task):
    """Retrieves the path of a task storage file in the temporary directory.

    Args:
      task_storage_format (str): storage format used to store task results.
      task (Task): task the storage changes are part of.

    Returns:
      str: path of a task storage file in the temporary directory or None if
          not set.
    """
    if task_storage_format == definitions.STORAGE_FORMAT_SQLITE:
      filename = '{0:s}.plaso'.format(task.identifier)
      return os.path.join(self._task_storage_path, filename)

    return None



# -*- coding: utf-8 -*-
"""Text parser plugin for Apache access log (access.log) files.

Parser based on the two default apache formats, common and combined log format
defined in https://httpd.apache.org/docs/2.4/logs.html
"""

import pyparsing

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.lib import errors
from plaso.parsers import text_parser
from plaso.parsers.text_plugins import interface


class ApacheAccessLogEventData(events.EventData):
  """Apache access log event data.

  Attributes:
    http_request_referer (str): http request referer header information.
    http_request (str): first line of http request.
    http_request_user_agent (str): http request user agent header information.
    http_response_bytes (int): http response bytes size without headers.
    http_response_code (int): http response code from server.
    ip_address (str): IPv4 or IPv6 addresses.
    port_number (int): canonical port of the server serving the request.
    recorded_time (dfdatetime.DateTimeValues): date and time the log entry
        was recorded.
    remote_name (str): remote logname (from identd, if supplied).
    server_name (str): canonical hostname of the server serving the request.
    user_name (str): logged user name.
  """

  DATA_TYPE = 'apache:access_log:entry'

  def __init__(self):
    """Initializes event data."""
    super(ApacheAccessLogEventData, self).__init__(data_type=self.DATA_TYPE)
    self.http_request = None
    self.http_request_referer = None
    self.http_request_user_agent = None
    self.http_response_bytes = None
    self.http_response_code = None
    self.ip_address = None
    self.port_number = None
    self.recorded_time = None
    self.remote_name = None
    self.server_name = None
    self.user_name = None


class ApacheAccessLogTextPlugin(interface.TextPlugin):
  """Text parser plugin for Apache access log (access.log) files."""

  NAME = 'apache_access'
  DATA_FORMAT = 'Apache access log (access.log) file'

  _MONTH_DICT = {
      'jan': 1,
      'feb': 2,
      'mar': 3,
      'apr': 4,
      'may': 5,
      'jun': 6,
      'jul': 7,
      'aug': 8,
      'sep': 9,
      'oct': 10,
      'nov': 11,
      'dec': 12}

  _INTEGER = pyparsing.Word(pyparsing.nums).setParseAction(
      lambda tokens: int(tokens[0], 10))

  _TWO_DIGITS = pyparsing.Word(pyparsing.nums, exact=2).setParseAction(
      lambda tokens: int(tokens[0], 10))

  _FOUR_DIGITS = pyparsing.Word(pyparsing.nums, exact=4).setParseAction(
      lambda tokens: int(tokens[0], 10))

  _THREE_LETTERS = pyparsing.Word(pyparsing.alphas, exact=3)

  _TIME_ZONE_OFFSET = (
      pyparsing.Word('+-', exact=1) + _TWO_DIGITS + _TWO_DIGITS)

  # Date and time values are formatted as: [18/Sep/2011:19:18:28 -0400]
  _DATE_TIME = pyparsing.Group(
      pyparsing.Suppress('[') + _TWO_DIGITS +
      pyparsing.Suppress('/') + _THREE_LETTERS +
      pyparsing.Suppress('/') + _FOUR_DIGITS +
      pyparsing.Suppress(':') + _TWO_DIGITS +
      pyparsing.Suppress(':') + _TWO_DIGITS +
      pyparsing.Suppress(':') + _TWO_DIGITS +
      _TIME_ZONE_OFFSET + pyparsing.Suppress(']')).setResultsName('date_time')

  _HTTP_METHOD = pyparsing.oneOf([
      'CONNECT', 'DELETE', 'GET', 'HEAD', 'OPTIONS', 'PATCH', 'POST', 'PUT',
      'TRACE'])

  _HTTP_VERSION = pyparsing.Combine(
      pyparsing.Literal('HTTP/') + pyparsing.Word(pyparsing.nums + '.'))

  _HTTP_REQUEST = pyparsing.Suppress('"') + pyparsing.Group(
      _HTTP_METHOD + pyparsing.Regex(r'\S*') + _HTTP_VERSION).setResultsName(
          'http_request') + pyparsing.Suppress('"')

  _IP_ADDRESS = (
      pyparsing.pyparsing_common.ipv4_address |
      pyparsing.pyparsing_common.ipv6_address)

  _REMOTE_NAME = (
      pyparsing.Word(pyparsing.alphanums) |
      pyparsing.Literal('-')).setResultsName('remote_name')

  _RESPONSE_BYTES = (
      pyparsing.Literal('-') | _INTEGER).setResultsName('response_bytes')

  _SERVER_NAME = (
      pyparsing.Word(pyparsing.alphanums + '-' + '.').setResultsName(
          'server_name'))

  _USER_AGENT = (
      pyparsing.Suppress('"') +
      pyparsing.CharsNotIn('"').setResultsName('user_agent') +
      pyparsing.Suppress('"'))

  _USER_NAME = (
      pyparsing.Word(pyparsing.alphanums + '@' + pyparsing.alphanums + '.') |
      pyparsing.Word(pyparsing.alphanums) |
      pyparsing.Literal('-')).setResultsName('user_name')

  _END_OF_LINE = pyparsing.Suppress(pyparsing.LineEnd())

  # Defined in https://httpd.apache.org/docs/2.4/logs.html
  # format: "%h %l %u %t \"%r\" %>s %b"
  _COMMON_LOG_FORMAT_LINE = (
      _IP_ADDRESS.setResultsName('ip_address') +
      _REMOTE_NAME +
      _USER_NAME +
      _DATE_TIME +
      _HTTP_REQUEST +
      _INTEGER.setResultsName('response_code') +
      _RESPONSE_BYTES +
      _END_OF_LINE)

  # Defined in https://httpd.apache.org/docs/2.4/logs.html
  # format: "%h %l %u %t \"%r\" %>s %b \"%{Referer}i\" \"%{User-agent}i\""
  _COMBINED_LOG_FORMAT_LINE = (
      _IP_ADDRESS.setResultsName('ip_address') +
      _REMOTE_NAME +
      _USER_NAME +
      _DATE_TIME +
      _HTTP_REQUEST +
      _INTEGER.setResultsName('response_code') +
      _RESPONSE_BYTES +
      pyparsing.QuotedString('"').setResultsName('referer') +
      _USER_AGENT +
      _END_OF_LINE)

  # "vhost_combined" format as used by Debian and related distributions.
  # "%v:%p %h %l %u %t \"%r\" %>s %b \"%{Referer}i\" \"%{User-Agent}i\""
  _VHOST_COMBINED_LOG_FORMAT_LINE = (
      _SERVER_NAME +
      pyparsing.Suppress(':') +
      _INTEGER.setResultsName('port_number') +
      _IP_ADDRESS.setResultsName('ip_address') +
      _REMOTE_NAME +
      _USER_NAME +
      _DATE_TIME +
      _HTTP_REQUEST +
      _INTEGER.setResultsName('response_code') +
      _RESPONSE_BYTES +
      pyparsing.QuotedString('"').setResultsName('referer') +
      _USER_AGENT +
      _END_OF_LINE)

  _LINE_STRUCTURES = [
      ('combined_log_format', _COMBINED_LOG_FORMAT_LINE),
      ('common_log_format', _COMMON_LOG_FORMAT_LINE),
      ('vhost_combined_log_format', _VHOST_COMBINED_LOG_FORMAT_LINE)]

  VERIFICATION_GRAMMAR = (
      _COMBINED_LOG_FORMAT_LINE ^ _COMMON_LOG_FORMAT_LINE ^
      _VHOST_COMBINED_LOG_FORMAT_LINE)

  VERIFICATION_LITERALS = [
      '"CONNECT ', '"DELETE ', '"GET ', '"HEAD ', ' HTTP/', '"OPTIONS ',
      '"PATCH ', '"POST ', '"PUT ', '"TRACE ']

  def _ParseRecord(self, parser_mediator, key, structure):
    """Parses a pyparsing structure.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): tokens from a parsed log line.

    Raises:
      ParseError: if the structure cannot be parsed.
    """
    time_elements_structure = self._GetValueFromStructure(
        structure, 'date_time')

    http_request = self._GetValueFromStructure(structure, 'http_request')
    if http_request:
      http_request = ' '.join(http_request)

    remote_name = self._GetValueFromStructure(structure, 'remote_name')
    if remote_name == '-':
      remote_name = None

    user_name = self._GetValueFromStructure(structure, 'user_name')
    if user_name == '-':
      user_name = None

    event_data = ApacheAccessLogEventData()
    event_data.http_request = http_request
    event_data.http_response_bytes = self._GetValueFromStructure(
        structure, 'response_bytes')
    event_data.http_response_code = self._GetValueFromStructure(
        structure, 'response_code')
    event_data.ip_address = self._GetValueFromStructure(structure, 'ip_address')
    event_data.recorded_time = self._ParseTimeElements(time_elements_structure)
    event_data.remote_name = remote_name
    event_data.user_name = user_name

    if key in ('combined_log_format', 'vhost_combined_log_format'):
      referer = self._GetValueFromStructure(structure, 'referer')
      if referer == '-':
        referer = None

      event_data.http_request_referer = referer
      event_data.http_request_user_agent = self._GetValueFromStructure(
          structure, 'user_agent')

    if key == 'vhost_combined_log_format':
      event_data.port_number = self._GetValueFromStructure(
          structure, 'port_number')
      event_data.server_name = self._GetValueFromStructure(
          structure, 'server_name')

    parser_mediator.ProduceEventData(event_data)

  def _ParseTimeElements(self, time_elements_structure):
    """Parses date and time elements of a log line.

    Args:
      time_elements_structure (pyparsing.ParseResults): date and time elements
          of a log line.

    Returns:
      dfdatetime.TimeElements: date and time value.

    Raises:
      ParseError: if a valid date and time value cannot be derived from
          the time elements.
    """
    try:
      (day_of_month, month_string, year, hours, minutes, seconds,
       time_zone_sign, time_zone_hours, time_zone_minutes) = (
          time_elements_structure)

      month = self._MONTH_DICT.get(month_string.lower(), 0)

      time_zone_offset = (time_zone_hours * 60) + time_zone_minutes
      if time_zone_sign == '-':
        time_zone_offset *= -1

      time_elements_tuple = (year, month, day_of_month, hours, minutes, seconds)
      return dfdatetime_time_elements.TimeElements(
          time_elements_tuple=time_elements_tuple,
          time_zone_offset=time_zone_offset)

    except (TypeError, ValueError) as exception:
      raise errors.ParseError(
          'Unable to parse time elements with error: {0!s}'.format(exception))

  def CheckRequiredFormat(self, parser_mediator, text_reader):
    """Check if the log record has the minimal structure required by the plugin.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      text_reader (EncodedTextReader): text reader.

    Returns:
      bool: True if this is the correct parser, False otherwise.
    """
    try:
      structure = self._VerifyString(text_reader.lines)
    except errors.ParseError:
      return False

    time_elements_structure = self._GetValueFromStructure(
        structure, 'date_time')

    try:
      self._ParseTimeElements(time_elements_structure)
    except errors.ParseError:
      return False

    return True


text_parser.TextLogParser.RegisterPlugin(ApacheAccessLogTextPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Dropbox sync_history database plugin."""

import unittest

from plaso.parsers.sqlite_plugins import dropbox

from tests.parsers.sqlite_plugins import test_lib


class DropboxSyncHistoryPluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the Dropbox sync_history database plugin."""

  def testProcess(self):
    """Tests the Process function on a Dropbox sync_history database file."""
    plugin = dropbox.DropboxSyncDatabasePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['sync_history.db'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 6)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'dropbox:sync_history:entry',
        'direction': 'upload',
        'event_type': 'file',
        'file_event_type': 'add',
        'file_identifier': 'XXXXXXXXXXXAAAAAAAAAGg',
        'local_path': '/home/useraa/Dropbox/loc1/create_local.txt',
        'recorded_time': '2022-02-17T10:57:18+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the MacOS application usage database plugin."""

import unittest

from plaso.parsers.sqlite_plugins import macos_appusage

from tests.parsers.sqlite_plugins import test_lib


class MacOSApplicationUsagePluginTest(test_lib.SQLitePluginTestCase):
  """Tests for the MacOS application usage activity database plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = macos_appusage.MacOSApplicationUsagePlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['application_usage.sqlite'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 5)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'application': '/Applications/Safari.app',
        'application_version': '9537.75.14',
        'bundle_identifier': 'com.apple.Safari',
        'count': 1,
        'data_type': 'macos:application_usage:entry',
        'event': 'launch',
        'last_used_time': '2014-05-07T18:52:02+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the preprocess mediator."""

import unittest

from plaso.containers import artifacts
from plaso.preprocessors import mediator
from plaso.storage.fake import writer as fake_writer

from tests import test_lib as shared_test_lib


class PreprocessMediatorTest(shared_test_lib.BaseTestCase):
  """Tests for the preprocess mediator."""

  # pylint: disable=protected-access

  def testAddTimeZoneInformation(self):
    """Tests the AddTimeZoneInformation function."""
    storage_writer = fake_writer.FakeStorageWriter()
    test_mediator = mediator.PreprocessMediator(storage_writer)

    time_zone_artifact = artifacts.TimeZoneArtifact(
        localized_name='Eastern (standaardtijd)', mui_form='@tzres.dll,-112',
        name='Eastern Standard Time')

    storage_writer.Open()

    try:
      test_mediator.AddTimeZoneInformation(time_zone_artifact)

      with self.assertRaises(KeyError):
        test_mediator.AddTimeZoneInformation(time_zone_artifact)

    finally:
      storage_writer.Close()

  def testProducePreprocessingWarning(self):
    """Tests the ProducePreprocessingWarning method."""
    storage_writer = fake_writer.FakeStorageWriter()
    test_mediator = mediator.PreprocessMediator(storage_writer)

    storage_writer.Open()

    try:
      test_mediator.ProducePreprocessingWarning(
          'test_plugin', 'test message')
    finally:
      storage_writer.Close()

  def testSetTimeZone(self):
    """Tests the SetTimeZone function."""
    storage_writer = fake_writer.FakeStorageWriter()
    test_mediator = mediator.PreprocessMediator(storage_writer)

    time_zone_artifact = artifacts.TimeZoneArtifact(
        localized_name='Eastern (standaardtijd)', mui_form='@tzres.dll,-112',
        name='Eastern Standard Time')

    storage_writer.Open()

    try:
      test_mediator.AddTimeZoneInformation(time_zone_artifact)
    finally:
      storage_writer.Close()

    # Set an IANA time zone name.
    test_mediator.SetTimeZone('Europe/Zurich')
    self.assertEqual(test_mediator.time_zone.zone, 'Europe/Zurich')

    # Set a Windows time zone name.
    test_mediator.SetTimeZone('Eastern Standard Time')
    self.assertEqual(test_mediator.time_zone.zone, 'America/New_York')

    # Set a localized Windows time zone name.
    test_mediator.SetTimeZone('Eastern (standaardtijd)')
    self.assertEqual(test_mediator.time_zone.zone, 'America/New_York')

    # Set a MUI form Windows time zone name.
    test_mediator.SetTimeZone('@tzres.dll,-112')
    self.assertEqual(test_mediator.time_zone.zone, 'America/New_York')

    with self.assertRaises(ValueError):
      test_mediator.SetTimeZone('Bogus')


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for Confluence access log text parser plugin."""

import unittest

from plaso.parsers.text_plugins import confluence_access

from tests.parsers.text_plugins import test_lib


class ConfluenceAccessTextPluginTest(test_lib.TextPluginTestCase):
  """Tests for Confluence access log text parser plugin."""

  def testProcessWithPre711AccessLog(self):
    """Tests the Process function with a pre 7.11 access log."""
    plugin = confluence_access.ConfluenceAccessTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['confluence_access.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 3)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'confluence:access',
        'forwarded_for': None,
        'http_request_referer': 'http://localhost/',
        'http_request_method': 'GET',
        'http_request_uri': '/index.action',
        'http_response_bytes': 7881,
        'http_response_code': 200,
        'http_request_user_agent': (
            'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) '
            'AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 '
            'Safari/537.36'),
        'http_version': 'HTTP/1.1',
        'process_duration': 1020,
        'recorded_time': '2021-06-17T12:57:26+02:00',
        'remote_name': '192.168.192.1',
        'thread_name': 'http-nio-8080-exec-6',
        'user_name': 'user'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testProcessWithPost711AccessLog(self):
    """Tests the Process function with a post 7.11 access log."""
    plugin = confluence_access.ConfluenceAccessTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['confluence_access_post7.11.log'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'confluence:access',
        'forwarded_for': '192.168.1.15',
        'http_request_referer': 'http://localhost/',
        'http_request_method': 'GET',
        'http_request_uri': '/index.action',
        'http_response_bytes': 7881,
        'http_response_code': 200,
        'http_request_user_agent': (
            'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) '
            'AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 '
            'Safari/537.36'),
        'http_version': 'HTTP/1.1',
        'process_duration': 1020,
        'recorded_time': '2021-06-17T12:57:26+02:00',
        'remote_name': '0:0:0:0:0:0:0:1',
        'thread_name': 'http-nio-8080-exec-6',
        'user_name': 'testuser'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Text parser plugin for Windows Firewall Log files."""

import pyparsing

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.lib import errors
from plaso.parsers import text_parser
from plaso.parsers.text_plugins import interface


class WinFirewallEventData(events.EventData):
  """Windows Firewall event data.

  Attributes:
    action (str): action taken.
    destination_ip (str): destination IP address.
    destination_port (int): TCP or UDP destination port.
    icmp_code (int): ICMP code.
    icmp_type (int): ICMP type.
    information (str): additional information.
    last_written_time (dfdatetime.DateTimeValues): entry last written date and
        time.
    packet_size (int): packet size.
    path (str): direction of the communication, which can be: SEND, RECEIVE,
        FORWARD, and UNKNOWN.
    protocol (str): IP protocol.
    source_ip (str): source IP address.
    source_port (int): TCP or UDP source port.
    tcp_ack (int): TCP acknowledgement number.
    tcp_flags (str): TCP flags.
    tcp_sequence_number (int): TCP sequence number.
    tcp_window_size (int): TCP window size.
  """

  DATA_TYPE = 'windows:firewall_log:entry'

  def __init__(self):
    """Initializes event data."""
    super(WinFirewallEventData, self).__init__(data_type=self.DATA_TYPE)
    self.action = None
    self.destination_ip = None
    self.destination_port = None
    self.icmp_code = None
    self.icmp_type = None
    self.information = None
    self.last_written_time = None
    self.packet_size = None
    self.path = None
    self.protocol = None
    self.source_ip = None
    self.source_port = None
    self.tcp_ack = None
    self.tcp_flags = None
    self.tcp_sequence_number = None
    self.tcp_window_size = None


class WinFirewallLogTextPlugin(interface.TextPlugin):
  """Text parser plugin for Windows Firewall Log files."""

  NAME = 'winfirewall'
  DATA_FORMAT = 'Windows Firewall log file'

  # A Windows Firewall is encoded using the system codepage.
  ENCODING = None

  _TWO_DIGITS = pyparsing.Word(pyparsing.nums, exact=2).setParseAction(
      lambda tokens: int(tokens[0], 10))

  _FOUR_DIGITS = pyparsing.Word(pyparsing.nums, exact=4).setParseAction(
      lambda tokens: int(tokens[0], 10))

  _DATE = pyparsing.Group(
      _FOUR_DIGITS + pyparsing.Suppress('-') +
      _TWO_DIGITS + pyparsing.Suppress('-') + _TWO_DIGITS)

  _TIME = pyparsing.Group(
      _TWO_DIGITS + pyparsing.Suppress(':') +
      _TWO_DIGITS + pyparsing.Suppress(':') + _TWO_DIGITS)

  _ACTION = pyparsing.Word(pyparsing.alphanums + '-', min=2)

  _WORD_OR_BLANK = (
      pyparsing.Word(pyparsing.alphanums) | pyparsing.Suppress('-'))

  _IP_ADDRESS_OR_BLANK = (
      pyparsing.pyparsing_common.ipv4_address |
      pyparsing.pyparsing_common.ipv6_address | pyparsing.Suppress('-'))

  _PORT_NUMBER_OR_BLANK = (
      pyparsing.Word(pyparsing.nums, max=6).setParseAction(
          lambda tokens: int(tokens[0], 10)) | pyparsing.Suppress('-'))

  _INTEGER_OR_BLANK = (
      pyparsing.Word(pyparsing.nums).setParseAction(
          lambda tokens: int(tokens[0], 10)) | pyparsing.Suppress('-'))

  _END_OF_LINE = pyparsing.Suppress(pyparsing.LineEnd())

  _FIELDS_METADATA = (
      pyparsing.Suppress('Fields: ') +
      pyparsing.restOfLine().setResultsName('fields'))

  _TIME_FORMAT_METADATA = (
      pyparsing.Suppress('Time Format: ') +
      pyparsing.restOfLine().setResultsName('time_format'))

  _METADATA = (
      _FIELDS_METADATA | _TIME_FORMAT_METADATA | pyparsing.restOfLine())

  _COMMENT_LOG_LINE = pyparsing.Suppress('#') + _METADATA + _END_OF_LINE

  # Version 1.5 fields:
  # date time action protocol src-ip dst-ip src-port dst-port size tcpflags
  # tcpsyn tcpack tcpwin icmptype icmpcode info path

  _LOG_LINE_1_5 = (
      _DATE.setResultsName('date') +
      _TIME.setResultsName('time') +
      _ACTION.setResultsName('action') +
      _WORD_OR_BLANK.setResultsName('protocol') +
      _IP_ADDRESS_OR_BLANK.setResultsName('source_ip') +
      _IP_ADDRESS_OR_BLANK.setResultsName('destination_ip') +
      _PORT_NUMBER_OR_BLANK.setResultsName('source_port') +
      _PORT_NUMBER_OR_BLANK.setResultsName('destination_port') +
      _INTEGER_OR_BLANK.setResultsName('packet_size') +
      _WORD_OR_BLANK.setResultsName('tcp_flags') +
      _INTEGER_OR_BLANK.setResultsName('tcp_sequence_number') +
      _INTEGER_OR_BLANK.setResultsName('tcp_ack') +
      _INTEGER_OR_BLANK.setResultsName('tcp_window_size') +
      _INTEGER_OR_BLANK.setResultsName('icmp_type') +
      _INTEGER_OR_BLANK.setResultsName('icmp_code') +
      _WORD_OR_BLANK.setResultsName('information') +
      _WORD_OR_BLANK.setResultsName('path') +
      _END_OF_LINE)

  # Common fields. Set results name with underscores, not hyphens because regex
  # will not pick them up.

  _LOG_LINE_STRUCTURES = {
      'action': _ACTION.setResultsName('action'),
      'date': _DATE.setResultsName('date'),
      'dst-ip': _IP_ADDRESS_OR_BLANK.setResultsName('destination_ip'),
      'dst-port': _PORT_NUMBER_OR_BLANK.setResultsName('destination_port'),
      'icmpcode': _INTEGER_OR_BLANK.setResultsName('icmp_code'),
      'icmptype': _INTEGER_OR_BLANK.setResultsName('icmp_type'),
      'info': _WORD_OR_BLANK.setResultsName('information'),
      'path': _WORD_OR_BLANK.setResultsName('path'),
      'protocol': _WORD_OR_BLANK.setResultsName('protocol'),
      'size': _INTEGER_OR_BLANK.setResultsName('packet_size'),
      'src-ip': _IP_ADDRESS_OR_BLANK.setResultsName('source_ip'),
      'src-port': _PORT_NUMBER_OR_BLANK.setResultsName('source_port'),
      'tcpack': _INTEGER_OR_BLANK.setResultsName('tcp_ack'),
      'tcpflags': _WORD_OR_BLANK.setResultsName('tcp_flags'),
      'tcpsyn': _INTEGER_OR_BLANK.setResultsName('tcp_sequence_number'),
      'tcpwin': _INTEGER_OR_BLANK.setResultsName('tcp_window_size'),
      'time': _TIME.setResultsName('time')}

  _HEADER_GRAMMAR = pyparsing.OneOrMore(_COMMENT_LOG_LINE)

  _LINE_STRUCTURES = [('log_line', _LOG_LINE_1_5)]

  VERIFICATION_GRAMMAR = (
      pyparsing.ZeroOrMore(
          pyparsing.Regex('#(Fields|Time Format|Version): .*') + _END_OF_LINE) +
      pyparsing.Regex('#Software: Microsoft Windows Firewall') + _END_OF_LINE)

  VERIFICATION_LITERALS = ['#Software: Microsoft Windows Firewall ']

  def __init__(self):
    """Initializes a text parser plugin."""
    super(WinFirewallLogTextPlugin, self).__init__()
    self._use_local_time = False

  def _ParseFieldsMetadata(self, parser_mediator, fields):
    """Parses the fields metadata and updates the log line definition to match.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      fields (str): field definitions.
    """
    log_line_structure = pyparsing.Empty()
    for member in fields.split(' '):
      if not member:
        continue

      field_structure = self._LOG_LINE_STRUCTURES.get(member, None)
      if not field_structure:
        field_structure = self._WORD_OR_BLANK
        parser_mediator.ProduceExtractionWarning((
            'missing definition for field: {0:s} defaulting to '
            'WORD_OR_BLANK').format(member))

      log_line_structure += field_structure

    log_line_structure += self._END_OF_LINE

    self._SetLineStructures([('log_line', log_line_structure)])

  def _ParseHeader(self, parser_mediator, text_reader):
    """Parses a text-log file header.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      text_reader (EncodedTextReader): text reader.

    Raises:
      ParseError: when the header cannot be parsed.
    """
    try:
      structure_generator = self._HEADER_GRAMMAR.scanString(
          text_reader.lines, maxMatches=1)
      structure, start, end = next(structure_generator)

    except StopIteration:
      structure = None

    except pyparsing.ParseException as exception:
      raise errors.ParseError(exception)

    if not structure or start != 0:
      raise errors.ParseError('No match found.')

    fields = self._GetValueFromStructure(structure, 'fields', default_value='')
    fields = fields.strip()
    if fields:
      self._ParseFieldsMetadata(parser_mediator, fields)

    time_format = self._GetValueFromStructure(
        structure, 'time_format', default_value='')
    self._use_local_time = time_format.lower() == 'local'

    text_reader.SkipAhead(end)

  def _ParseLogLine(self, parser_mediator, structure):
    """Parse a single log line.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      structure (pyparsing.ParseResults): tokens from a parsed log line.
    """
    event_data = WinFirewallEventData()
    event_data.action = self._GetValueFromStructure(structure, 'action')
    event_data.destination_ip = self._GetValueFromStructure(
        structure, 'destination_ip')
    event_data.destination_port = self._GetValueFromStructure(
        structure, 'destination_port')
    event_data.icmp_code = self._GetValueFromStructure(structure, 'icmp_code')
    event_data.icmp_type = self._GetValueFromStructure(structure, 'icmp_type')
    event_data.information = self._GetValueFromStructure(
        structure, 'information')
    event_data.last_written_time = self._ParseTimeElements(structure)
    event_data.path = self._GetValueFromStructure(structure, 'path')
    event_data.protocol = self._GetValueFromStructure(structure, 'protocol')
    event_data.packet_size = self._GetValueFromStructure(
        structure, 'packet_size')
    event_data.source_ip = self._GetValueFromStructure(structure, 'source_ip')
    event_data.source_port = self._GetValueFromStructure(
        structure, 'source_port')
    event_data.tcp_ack = self._GetValueFromStructure(structure, 'tcp_ack')
    event_data.tcp_flags = self._GetValueFromStructure(structure, 'tcp_flags')
    event_data.tcp_sequence_number = self._GetValueFromStructure(
        structure, 'tcp_sequence_number')
    event_data.tcp_window_size = self._GetValueFromStructure(
        structure, 'tcp_window_size')

    parser_mediator.ProduceEventData(event_data)

  def _ParseRecord(self, parser_mediator, key, structure):
    """Parses a pyparsing structure.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): tokens from a parsed log line.

    Raises:
      ParseError: if the structure cannot be parsed.
    """
    self._ParseLogLine(parser_mediator, structure)

  def _ParseTimeElements(self, structure):
    """Parses date and time elements of a log line.

    Args:
      structure (pyparsing.ParseResults): tokens from a parsed log line.

    Returns:
      dfdatetime.TimeElements: date and time value.

    Raises:
      ParseError: if a valid date and time value cannot be derived from
          the time elements.
    """
    try:
      date_elements_structure = self._GetValueFromStructure(structure, 'date')
      time_elements_structure = self._GetValueFromStructure(structure, 'time')

      year, month, day_of_month = date_elements_structure
      hours, minutes, seconds = time_elements_structure

      time_elements_tuple = (year, month, day_of_month, hours, minutes, seconds)

      date_time = dfdatetime_time_elements.TimeElements(
          time_elements_tuple=time_elements_tuple)
      date_time.is_local_time = self._use_local_time

      return date_time

    except (TypeError, ValueError) as exception:
      raise errors.ParseError(
          'Unable to parse time elements with error: {0!s}'.format(exception))

  def _ResetState(self):
    """Resets stored values."""
    self._use_local_time = False

    self._SetLineStructures(self._LINE_STRUCTURES)

  def CheckRequiredFormat(self, parser_mediator, text_reader):
    """Check if the log record has the minimal structure required by the plugin.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      text_reader (EncodedTextReader): text reader.

    Returns:
      bool: True if this is the correct parser, False otherwise.
    """
    try:
      self._VerifyString(text_reader.lines)
    except errors.ParseError:
      return False

    self._ResetState()

    return True


text_parser.TextLogParser.RegisterPlugin(WinFirewallLogTextPlugin)



# -*- coding: utf-8 -*-
"""Bencode plugin related functions and classes for testing."""

from tests.parsers import test_lib


class BencodePluginTestCase(test_lib.ParserTestCase):
  """Bencode parser plugin test case."""
